# -*- coding: utf-8 -*-
"""unit testing"""

from __future__ import unicode_literals

from decimal import Decimal

from django.core.urlresolvers import reverse
from django.contrib.auth.models import User

from coop_cms.tests import BeautifulSoup
from model_mommy import mommy

from balafon.unit_tests import TestCase
from balafon.Crm.models import Action, ActionMenu, ActionType, Contact, ActionStatus, MailtoSettings
from balafon.Store import models


class StockThresholdTest(TestCase):
    """It should display a warning sign when stock is below threshold"""

    def test_above_threshold(self):
        """It should not show a warning sign"""
        item = mommy.make(models.StoreItem, stock_count=40, stock_threshold=30)
        soup = BeautifulSoup(item.stock_threshold_alert())
        self.assertEqual(0, len(soup.select("img")))

    def test_threshold_not_defined(self):
        """It should not show a warning sign"""
        item = mommy.make(models.StoreItem, stock_count=40, stock_threshold=None)
        self.assertEqual("", item.stock_threshold_alert())

    def test_stock_not_defined(self):
        """It should not show a warning sign"""
        item = mommy.make(models.StoreItem, stock_count=None, stock_threshold=None)
        self.assertEqual("", item.stock_threshold_alert())

    def test_below_threshold(self):
        """It should show a warning sign"""
        item = mommy.make(models.StoreItem, stock_count=20, stock_threshold=30)
        soup = BeautifulSoup(item.stock_threshold_alert())
        self.assertEqual(1, len(soup.select("img")))


class VatInclusivePriceTest(TestCase):
    """It should calculate price with VAT"""

    def test_price(self):
        """it should be right"""
        vat_rate = mommy.make(models.VatRate, rate=Decimal("10.0"))
        item = mommy.make(models.StoreItem, pre_tax_price=Decimal("20.0"), vat_rate=vat_rate)
        self.assertEqual(Decimal("22.0"), item.vat_incl_price())

    def test_sale_no_vat(self):
        """Vat should be 0"""
        sale = mommy.make(models.Sale)
        item = mommy.make(models.SaleItem, quantity=Decimal(2), pre_tax_price=Decimal("10"), vat_rate=None, sale=sale)

        self.assertEqual(item.vat_price(), Decimal(0))
        self.assertEqual(item.total_vat_price(), Decimal(0))
        self.assertEqual(item.vat_incl_price(), Decimal("10"))
        self.assertEqual(item.vat_incl_total_price(), Decimal("20"))

    def test_sale_with_vat(self):
        """Vat should not be 0"""
        vat_rate = mommy.make(models.VatRate, rate=Decimal("10.0"))
        sale = mommy.make(models.Sale)
        item = mommy.make(models.SaleItem, quantity=Decimal(2), pre_tax_price=Decimal("11"), vat_rate=vat_rate, sale=sale)

        self.assertEqual(item.vat_price(), Decimal("1.1"))
        self.assertEqual(item.total_vat_price(), Decimal("2.2"))
        self.assertEqual(item.vat_incl_price(), Decimal("12.1"))
        self.assertEqual(item.vat_incl_total_price(), Decimal("24.2"))


class StoreManagementActionTypeTest(TestCase):
    """It should create actions menus"""

    def test_create_store_action_type(self):
        """It should create the menu"""
        self.assertEqual(0, ActionMenu.objects.count())
        action_type = mommy.make(ActionType)
        mommy.make(models.StoreManagementActionType, action_type=action_type)
        self.assertEqual(1, ActionMenu.objects.count())
        menu = ActionMenu.objects.all()[0]
        self.assertEqual(menu.view_name, 'store_view_sales_document')

    def test_create_store_action_type_and_save(self):
        """It should create only 1 menu"""
        self.assertEqual(0, ActionMenu.objects.count())
        action_type = mommy.make(ActionType)
        store_action_type = mommy.make(models.StoreManagementActionType, action_type=action_type)
        self.assertEqual(1, ActionMenu.objects.count())
        menu = ActionMenu.objects.all()[0]
        self.assertEqual(menu.view_name, 'store_view_sales_document')
        store_action_type.save()
        self.assertEqual(1, ActionMenu.objects.count())


class UpdateActionItemTest(TestCase):
    """It should keep Action amount up-to-date"""

    def test_amount_on_add(self):
        """It should set action amount correctly"""
        action_type = mommy.make(ActionType)
        mommy.make(
            models.StoreManagementActionType,
            action_type=action_type,
            show_amount_as_pre_tax=True
        )

        action = mommy.make(Action, type=action_type)

        self.assertNotEqual(action.sale, None)
        vat_rate = mommy.make(models.VatRate, rate="20.0")

        mommy.make(models.SaleItem, quantity=1, pre_tax_price="12.6", vat_rate=vat_rate, sale=action.sale)

        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.amount, Decimal("12.6"))

    def test_amount_on_update_quantity(self):
        """It should set action amount correctly"""
        action_type = mommy.make(ActionType)
        mommy.make(
            models.StoreManagementActionType,
            action_type=action_type,
            show_amount_as_pre_tax=True
        )

        action = mommy.make(Action, type=action_type)

        self.assertNotEqual(action.sale, None)
        vat_rate = mommy.make(models.VatRate, rate="20.0")

        mommy.make(models.SaleItem, quantity=1, pre_tax_price="12.6", vat_rate=vat_rate, sale=action.sale)
        sale_item2 = mommy.make(models.SaleItem, quantity=2, pre_tax_price="4", vat_rate=vat_rate, sale=action.sale)

        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.amount, Decimal("20.6"))

        sale_item2.quantity = 3
        sale_item2.save()

        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.amount, Decimal("24.6"))

    def test_amount_on_update_price(self):
        """It should set action amount correctly"""
        action_type = mommy.make(ActionType)
        mommy.make(
            models.StoreManagementActionType,
            action_type=action_type,
            show_amount_as_pre_tax=True
        )

        action = mommy.make(Action, type=action_type)

        self.assertNotEqual(action.sale, None)
        vat_rate = mommy.make(models.VatRate, rate="20.0")

        mommy.make(models.SaleItem, quantity=1, pre_tax_price="12.6", vat_rate=vat_rate, sale=action.sale)
        sale_item2 = mommy.make(models.SaleItem, quantity=2, pre_tax_price="4", vat_rate=vat_rate, sale=action.sale)

        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.amount, Decimal("20.6"))

        sale_item2.pre_tax_price = 2
        sale_item2.save()

        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.amount, Decimal("16.6"))

    def test_amount_on_delete_sale(self):
        """It should set action amount correctly"""
        action_type = mommy.make(ActionType)
        mommy.make(
            models.StoreManagementActionType,
            action_type=action_type,
            show_amount_as_pre_tax=True
        )

        action = mommy.make(Action, type=action_type)

        self.assertNotEqual(action.sale, None)
        vat_rate = mommy.make(models.VatRate, rate="20.0")

        mommy.make(models.SaleItem, quantity=1, pre_tax_price="12.6", vat_rate=vat_rate, sale=action.sale)
        sale_item2 = mommy.make(models.SaleItem, quantity=2, pre_tax_price="4", vat_rate=vat_rate, sale=action.sale)

        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.amount, Decimal("20.6"))

        sale_item2.delete()

        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.amount, Decimal("12.6"))

    def test_amount_on_vat(self):
        """It should set action amount correctly"""
        action_type = mommy.make(ActionType)
        mommy.make(
            models.StoreManagementActionType,
            action_type=action_type,
            show_amount_as_pre_tax=False
        )

        action = mommy.make(Action, type=action_type)

        self.assertNotEqual(action.sale, None)
        vat_rate1 = mommy.make(models.VatRate, rate="20.0")
        vat_rate2 = mommy.make(models.VatRate, rate="10.0")

        mommy.make(models.SaleItem, quantity=1, pre_tax_price="10", vat_rate=vat_rate1, sale=action.sale)
        mommy.make(models.SaleItem, quantity=2, pre_tax_price="1", vat_rate=vat_rate2, sale=action.sale)

        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.amount, Decimal("14.2"))

    def test_amount_on_update_vat(self):
        """It should set action amount correctly"""
        action_type = mommy.make(ActionType)
        mommy.make(
            models.StoreManagementActionType,
            action_type=action_type,
            show_amount_as_pre_tax=False
        )

        action = mommy.make(Action, type=action_type)

        self.assertNotEqual(action.sale, None)
        vat_rate1 = mommy.make(models.VatRate, rate="20.0")
        vat_rate2 = mommy.make(models.VatRate, rate="10.0")

        mommy.make(models.SaleItem, quantity=1, pre_tax_price="10", vat_rate=vat_rate1, sale=action.sale)
        sale_item2 = mommy.make(models.SaleItem, quantity=2, pre_tax_price="1", vat_rate=vat_rate2, sale=action.sale)

        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.amount, Decimal("14.2"))

        sale_item2.vat_rate = vat_rate1
        sale_item2.save()

        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.amount, Decimal("14.4"))


class SaleTotalTest(TestCase):
    """test that the total amount of a sale is calculated correctly"""

    def test_amount_empty_cart(self):
        """It should return 0"""
        sale = mommy.make(models.Sale)
        self.assertEqual(Decimal(0), sale.vat_incl_total_price())

    def test_amount_cart(self):
        """It should return sum"""
        sale = mommy.make(models.Sale)
        vat_rate_20 = mommy.make(models.VatRate, rate="20.0")
        vat_rate_10 = mommy.make(models.VatRate, rate="10.0")

        mommy.make(models.SaleItem, quantity=1, pre_tax_price="10", vat_rate=vat_rate_20, sale=sale)
        mommy.make(models.SaleItem, quantity=2, pre_tax_price="1", vat_rate=vat_rate_10, sale=sale)
        mommy.make(models.SaleItem, quantity=1, pre_tax_price="5", vat_rate=vat_rate_10, sale=sale)

        self.assertEqual(Decimal("19.7"), sale.vat_incl_total_price())

    def test_vat_amount_empty_cart(self):
        """It should return empty list"""
        sale = mommy.make(models.Sale)
        self.assertEqual([], sale.vat_total_amounts())

    def test_vat_amount_cart(self):
        """It should return list with dictionary with total amount of vat per rate"""
        sale = mommy.make(models.Sale)
        vat_rate_20 = mommy.make(models.VatRate, rate="20.0")
        vat_rate_10 = mommy.make(models.VatRate, rate="10.0")

        mommy.make(models.SaleItem, quantity=1, pre_tax_price="10", vat_rate=vat_rate_20, sale=sale)
        mommy.make(models.SaleItem, quantity=2, pre_tax_price="1", vat_rate=vat_rate_10, sale=sale)
        mommy.make(models.SaleItem, quantity=1, pre_tax_price="5", vat_rate=vat_rate_10, sale=sale)

        vat_totals = sale.vat_total_amounts()
        self.assertEqual(len(vat_totals), 2)
        self.assertEqual(vat_totals[0]['vat_rate'], vat_rate_10)
        self.assertEqual(vat_totals[0]['amount'], Decimal("0.7"))
        self.assertEqual(vat_totals[1]['vat_rate'], vat_rate_20)
        self.assertEqual(vat_totals[1]['amount'], Decimal("2"))

    def test_amount_no_vat(self):
        """It should return list with no vat value"""
        sale = mommy.make(models.Sale)
        mommy.make(models.SaleItem, quantity=1, pre_tax_price="10", vat_rate=None, sale=sale)
        vat_totals = sale.vat_total_amounts()
        self.assertEqual(len(vat_totals), 0)


class CategoryNameTest(TestCase):
    """Test that category name can not be duplicated"""

    def test_strip_name(self):
        """it should strip name"""
        category = mommy.make(models.StoreItemCategory, name=' Abc ')
        self.assertEqual(category.name, 'Abc')

    def test_merge_duplicates_articles(self):
        """it should merge articles in only 1 category"""
        category_name = "Abc"
        category1 = mommy.make(models.StoreItemCategory, name=category_name)
        article1 = mommy.make(models.StoreItem, category=category1)
        article2 = mommy.make(models.StoreItem, category=category1)

        category2 = mommy.make(models.StoreItemCategory, name=category_name)

        category = models.StoreItemCategory.objects.get(name=category_name)
        self.assertEqual(category.id, category2.id)
        self.assertEqual(category.storeitem_set.count(), 2)
        self.assertTrue(article1 in category.storeitem_set.all())
        self.assertTrue(article2 in category.storeitem_set.all())

        self.assertEqual(models.StoreItemCategory.objects.filter(id=category1.id).count(), 0)

    def test_merge_duplicates_subcategories(self):
        """it should merge subcategories in only 1 category"""
        category_name = "Abc"
        category1 = mommy.make(models.StoreItemCategory, name=category_name)
        sub_category1 = mommy.make(models.StoreItemCategory, name='Def', parent=category1)
        sub_category2 = mommy.make(models.StoreItemCategory, name='Ghi', parent=category1)

        category2 = mommy.make(models.StoreItemCategory, name=category_name)

        category = models.StoreItemCategory.objects.get(name=category_name)
        self.assertEqual(category.id, category2.id)
        self.assertEqual(category.subcategories_set.count(), 2)
        self.assertTrue(sub_category1 in category.subcategories_set.all())
        self.assertTrue(sub_category2 in category.subcategories_set.all())

        self.assertEqual(models.StoreItemCategory.objects.filter(id=category1.id).count(), 0)

    def test_path_name(self):
        """it should show full name"""
        category_name = "Abc"
        category1 = mommy.make(models.StoreItemCategory, name=category_name)
        sub_category1 = mommy.make(models.StoreItemCategory, name='Def', parent=category1)

        self.assertEqual(category1.get_path_name(), category1.name)
        self.assertEqual(sub_category1.get_path_name(), '{0} > {1}'.format(category1.name, sub_category1.name))


class PricePolicyTest(TestCase):
    """Test that price is calculated properly"""

    def test_article_no_policy(self):
        """it should not change the price"""
        add_20_percent = mommy.make(models.PricePolicy, policy='multiply_purchase_by_ratio', parameters="1.2")

        category1 = mommy.make(models.StoreItemCategory, price_policy=add_20_percent)

        article = mommy.make(
            models.StoreItem,
            category=category1,
            price_policy=None,
            purchase_price=Decimal("1"),
            pre_tax_price=Decimal("2")
        )

        self.assertEqual(article.pre_tax_price, Decimal("2"))

    def test_category_no_policy(self):
        """it should not change the price"""
        from_category = mommy.make(models.PricePolicy, policy='from_category', parameters='')

        category1 = mommy.make(models.StoreItemCategory, price_policy=None)

        article = mommy.make(
            models.StoreItem,
            category=category1,
            price_policy=from_category,
            purchase_price=Decimal("1"),
            pre_tax_price=Decimal("2")
        )

        self.assertEqual(article.pre_tax_price, Decimal("2"))

    def test_article_ratio_policy(self):
        """it should change the price"""
        add_20_percent = mommy.make(models.PricePolicy, policy='multiply_purchase_by_ratio', parameters="1.2")
        add_50_percent = mommy.make(models.PricePolicy, policy='multiply_purchase_by_ratio', parameters="1.5")

        category1 = mommy.make(models.StoreItemCategory, price_policy=add_50_percent)

        article = mommy.make(
            models.StoreItem,
            category=category1,
            price_policy=add_20_percent,
            purchase_price=Decimal("1"),
            pre_tax_price=Decimal("2")
        )

        self.assertEqual(article.pre_tax_price, Decimal("1.2"))

    def test_category_ratio_policy(self):
        """it should change the price"""
        from_category = mommy.make(models.PricePolicy, policy='from_category', parameters='')
        add_50_percent = mommy.make(models.PricePolicy, policy='multiply_purchase_by_ratio', parameters="1.5")

        category1 = mommy.make(models.StoreItemCategory, price_policy=add_50_percent)

        article = mommy.make(
            models.StoreItem,
            category=category1,
            price_policy=from_category,
            purchase_price=Decimal("1"),
            pre_tax_price=Decimal("2")
        )

        self.assertEqual(article.pre_tax_price, Decimal("1.5"))

    def test_parent_category_ratio_policy(self):
        """it should change the price"""
        from_category = mommy.make(models.PricePolicy, policy='from_category', parameters='')
        add_20_percent = mommy.make(models.PricePolicy, policy='multiply_purchase_by_ratio', parameters="1.2")

        category1 = mommy.make(models.StoreItemCategory, price_policy=add_20_percent)
        category2 = mommy.make(models.StoreItemCategory, price_policy=from_category, parent=category1)

        article = mommy.make(
            models.StoreItem,
            category=category2,
            price_policy=from_category,
            purchase_price=Decimal("1"),
            pre_tax_price=Decimal("2")
        )

        self.assertEqual(article.pre_tax_price, Decimal("1.2"))

    def test_change_category_policy_update_price(self):
        """it should change the price"""
        from_category = mommy.make(models.PricePolicy, policy='from_category', parameters='')
        add_20_percent = mommy.make(models.PricePolicy, policy='multiply_purchase_by_ratio', parameters="1.2")
        add_50_percent = mommy.make(models.PricePolicy, policy='multiply_purchase_by_ratio', parameters="1.5")

        category1 = mommy.make(models.StoreItemCategory, price_policy=add_20_percent)
        category2 = mommy.make(models.StoreItemCategory, price_policy=from_category, parent=category1)

        article = mommy.make(
            models.StoreItem,
            category=category2,
            price_policy=from_category,
            purchase_price=Decimal("1"),
            pre_tax_price=Decimal("2")
        )

        self.assertEqual(article.pre_tax_price, Decimal("1.2"))

        category2.price_policy = add_50_percent
        category2.save()

        article = models.StoreItem.objects.get(id=article.id)
        self.assertEqual(article.pre_tax_price, Decimal("1.5"))

    def test_change_category_parent_policy_update_price(self):
        """it should change the price"""
        from_category = mommy.make(models.PricePolicy, policy='from_category', parameters='')
        add_20_percent = mommy.make(models.PricePolicy, policy='multiply_purchase_by_ratio', parameters="1.2")
        add_50_percent = mommy.make(models.PricePolicy, policy='multiply_purchase_by_ratio', parameters="1.5")

        category1 = mommy.make(models.StoreItemCategory, price_policy=add_20_percent)
        category2 = mommy.make(models.StoreItemCategory, price_policy=from_category, parent=category1)

        article = mommy.make(
            models.StoreItem,
            category=category2,
            price_policy=from_category,
            purchase_price=Decimal("1"),
            pre_tax_price=Decimal("2")
        )

        self.assertEqual(article.pre_tax_price, Decimal("1.2"))

        category1.price_policy = add_50_percent
        category1.save()

        article = models.StoreItem.objects.get(id=article.id)
        self.assertEqual(article.pre_tax_price, Decimal("1.5"))

    def test_category_parent_policy_not_set(self):
        """it should change the price"""
        from_category = mommy.make(models.PricePolicy, policy='from_category', parameters='')

        category1 = mommy.make(models.StoreItemCategory, price_policy=None)
        category2 = mommy.make(models.StoreItemCategory, price_policy=from_category, parent=category1)

        article = mommy.make(
            models.StoreItem,
            category=category2,
            price_policy=from_category,
            purchase_price=Decimal("1"),
            pre_tax_price=Decimal("2")
        )

        self.assertEqual(article.pre_tax_price, Decimal("2"))

    def test_category_invalid_purchase_prise(self):
        """it should not change the price"""
        from_category = mommy.make(models.PricePolicy, policy='from_category', parameters='')
        add_20_percent = mommy.make(models.PricePolicy, policy='multiply_purchase_by_ratio', parameters="1.2")

        category1 = mommy.make(models.StoreItemCategory, price_policy=None)
        category2 = mommy.make(models.StoreItemCategory, price_policy=from_category, parent=category1)

        article = mommy.make(
            models.StoreItem,
            category=category2,
            price_policy=add_20_percent,
            purchase_price=None,
            pre_tax_price=Decimal("2")
        )

        self.assertEqual(article.pre_tax_price, Decimal("2"))


class BlankLineTest(TestCase):
    """It should calculate price with VAT"""

    def test_blank_lines(self):
        """it should return all lines and calculate price properly"""
        vat_rate = mommy.make(models.VatRate, rate=Decimal("10.0"))
        sale = mommy.make(models.Sale)
        mommy.make(models.SaleItem, quantity=2, pre_tax_price="10", vat_rate=vat_rate, sale=sale)
        item2 = mommy.make(models.SaleItem, quantity=1, pre_tax_price="1", vat_rate=None, sale=sale, is_blank=True)
        mommy.make(models.SaleItem, quantity=1, pre_tax_price="15", vat_rate=vat_rate, sale=sale)

        self.assertEqual(sale.saleitem_set.count(), 3)
        self.assertEqual(item2.quantity, 0)
        self.assertEqual(item2.pre_tax_price, 0)

        self.assertEqual(Decimal("38.5"), sale.vat_incl_total_price())


class SaleDiscountTest(TestCase):
    """It should take discount into account when calculating price"""

    def test_calculate_discount(self):
        """It should have a discount"""
        vat_rate = mommy.make(models.VatRate, rate=Decimal("20.0"))

        tag = mommy.make(models.StoreItemTag, name="vrac")

        discount = mommy.make(
            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
        )
        discount.tags.add(tag)
        discount.save()

        item = mommy.make(models.StoreItem, pre_tax_price=Decimal("10"), vat_rate=vat_rate)
        item.tags.add(tag)
        item.save()

        sale = mommy.make(models.Sale)
        sale_item = mommy.make(
            models.SaleItem, quantity=2, item=item, pre_tax_price=item.pre_tax_price, vat_rate=item.vat_rate, sale=sale
        )
        sale.save()

        self.assertEqual(sale.saleitem_set.count(), 1)
        self.assertEqual(Decimal("18"), sale.pre_tax_total_price())
        self.assertEqual(Decimal("21.6"), sale.vat_incl_total_price())

        sale_item = models.SaleItem.objects.get(id=sale_item.id)
        self.assertEqual(sale_item.discount, discount)
        self.assertEqual(sale_item.unit_price(), Decimal('9'))
        self.assertEqual(sale_item.vat_incl_price(), Decimal('10.8'))
        self.assertEqual(sale_item.vat_price(), Decimal('1.8'))
        self.assertEqual(sale_item.total_vat_price(), Decimal('3.6'))

    def test_calculate_discount_price_class(self):
        """It should have a discount"""
        vat_rate = mommy.make(models.VatRate, rate=Decimal("20.0"))

        discount = mommy.make(
            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
        )
        price_class = mommy.make(models.PriceClass)
        price_class.discounts.add(discount)
        price_class.save()

        item = mommy.make(models.StoreItem, pre_tax_price=Decimal("10"), vat_rate=vat_rate, price_class=price_class)

        sale = mommy.make(models.Sale)
        sale_item = mommy.make(
            models.SaleItem, quantity=2, item=item, pre_tax_price=item.pre_tax_price, vat_rate=item.vat_rate, sale=sale
        )
        sale.save()

        self.assertEqual(sale.saleitem_set.count(), 1)
        self.assertEqual(Decimal("18"), sale.pre_tax_total_price())
        self.assertEqual(Decimal("21.6"), sale.vat_incl_total_price())

        sale_item = models.SaleItem.objects.get(id=sale_item.id)
        self.assertEqual(sale_item.discount, discount)
        self.assertEqual(sale_item.unit_price(), Decimal('9'))
        self.assertEqual(sale_item.vat_incl_price(), Decimal('10.8'))
        self.assertEqual(sale_item.vat_price(), Decimal('1.8'))
        self.assertEqual(sale_item.total_vat_price(), Decimal('3.6'))

    def test_calculate_discount_disabled(self):
        """It should not have a discount"""
        vat_rate = mommy.make(models.VatRate, rate=Decimal("20.0"))

        tag = mommy.make(models.StoreItemTag, name="vrac")

        discount = mommy.make(
            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=False
        )
        discount.tags.add(tag)
        discount.save()

        item = mommy.make(models.StoreItem, pre_tax_price=Decimal("10"), vat_rate=vat_rate)
        item.tags.add(tag)
        item.save()

        sale = mommy.make(models.Sale)
        sale_item = mommy.make(
            models.SaleItem, quantity=2, item=item, pre_tax_price=item.pre_tax_price, vat_rate=item.vat_rate, sale=sale
        )
        sale.save()

        self.assertEqual(sale.saleitem_set.count(), 1)
        self.assertEqual(Decimal("20"), sale.pre_tax_total_price())
        self.assertEqual(Decimal("24"), sale.vat_incl_total_price())

        sale_item = models.SaleItem.objects.get(id=sale_item.id)
        self.assertEqual(sale_item.discount, None)
        self.assertEqual(sale_item.unit_price(), Decimal('10'))
        self.assertEqual(sale_item.vat_incl_price(), Decimal('12'))
        self.assertEqual(sale_item.vat_price(), Decimal('2'))
        self.assertEqual(sale_item.total_vat_price(), Decimal('4'))

    def test_calculate_discount_quantity_zero(self):
        """It should not have a discount"""
        vat_rate = mommy.make(models.VatRate, rate=Decimal("20.0"))

        tag = mommy.make(models.StoreItemTag, name="vrac")

        discount = mommy.make(
            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=False
        )
        discount.tags.add(tag)
        discount.save()

        item = mommy.make(models.StoreItem, pre_tax_price=Decimal("10"), vat_rate=vat_rate)
        item.tags.add(tag)
        item.save()

        sale = mommy.make(models.Sale)
        sale_item = mommy.make(
            models.SaleItem, quantity=0, item=item, pre_tax_price=item.pre_tax_price, vat_rate=item.vat_rate, sale=sale
        )
        sale.save()

        self.assertEqual(sale.saleitem_set.count(), 1)
        self.assertEqual(Decimal(0), sale.pre_tax_total_price())
        self.assertEqual(Decimal(0), sale.vat_incl_total_price())

        sale_item = models.SaleItem.objects.get(id=sale_item.id)
        self.assertEqual(sale_item.discount, None)
        self.assertEqual(sale_item.unit_price(), Decimal("10"))
        self.assertEqual(sale_item.vat_incl_price(), Decimal("12"))
        self.assertEqual(sale_item.vat_price(), Decimal("2"))
        self.assertEqual(sale_item.total_vat_price(), Decimal(0))

    def test_several_discount(self):
        """It should have only 1 discount"""
        vat_rate = mommy.make(models.VatRate, rate=Decimal("20.0"))

        tag = mommy.make(models.StoreItemTag, name="vrac")

        discount = mommy.make(
            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
        )
        discount.tags.add(tag)
        discount.save()

        discount2 = mommy.make(
            models.Discount, name='-20% à partir de 2 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
        )
        discount2.tags.add(tag)
        discount2.save()

        item = mommy.make(models.StoreItem, pre_tax_price=Decimal("10"), vat_rate=vat_rate)
        item.tags.add(tag)
        item.save()

        sale = mommy.make(models.Sale)
        sale_item = mommy.make(
            models.SaleItem, quantity=2, item=item, pre_tax_price=item.pre_tax_price, vat_rate=item.vat_rate, sale=sale
        )
        sale.save()

        self.assertEqual(sale.saleitem_set.count(), 1)
        self.assertEqual(Decimal("18"), sale.pre_tax_total_price())
        self.assertEqual(Decimal("21.6"), sale.vat_incl_total_price())

        sale_item = models.SaleItem.objects.get(id=sale_item.id)
        self.assertEqual(sale_item.discount, discount)
        self.assertEqual(sale_item.unit_price(), Decimal('9'))
        self.assertEqual(sale_item.vat_incl_price(), Decimal('10.8'))
        self.assertEqual(sale_item.vat_price(), Decimal('1.8'))
        self.assertEqual(sale_item.total_vat_price(), Decimal('3.6'))

    def test_several_discount_right_quantity(self):
        """It should have only 1 discount"""
        vat_rate = mommy.make(models.VatRate, rate=Decimal("20.0"))

        tag = mommy.make(models.StoreItemTag, name="vrac")

        discount = mommy.make(
            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
        )
        discount.tags.add(tag)
        discount.save()

        discount2 = mommy.make(
            models.Discount, name='-20% à partir de 3 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
        )
        discount2.tags.add(tag)
        discount2.save()

        item = mommy.make(models.StoreItem, pre_tax_price=Decimal("10"), vat_rate=vat_rate)
        item.tags.add(tag)
        item.save()

        sale = mommy.make(models.Sale)
        sale_item = mommy.make(
            models.SaleItem, quantity=3, item=item, pre_tax_price=item.pre_tax_price, vat_rate=item.vat_rate, sale=sale
        )
        sale.save()

        self.assertEqual(sale.saleitem_set.count(), 1)
        self.assertEqual(Decimal("24"), sale.pre_tax_total_price())
        self.assertEqual(Decimal("28.8"), sale.vat_incl_total_price())

        sale_item = models.SaleItem.objects.get(id=sale_item.id)
        self.assertEqual(sale_item.discount, discount2)
        self.assertEqual(sale_item.unit_price(), Decimal('8'))
        self.assertEqual(sale_item.vat_incl_price(), Decimal('9.6'))
        self.assertEqual(sale_item.vat_price(), Decimal('1.6'))
        self.assertEqual(sale_item.total_vat_price(), Decimal('4.8'))

    def test_calculate_discount_on_item_saved(self):
        """It should recalculate discount when item is saved"""
        vat_rate = mommy.make(models.VatRate, rate=Decimal("20.0"))

        tag = mommy.make(models.StoreItemTag, name="vrac")

        discount = mommy.make(
            models.Discount, name='-10% à partir de 3 Kg', rate=Decimal("10"), quantity=Decimal("3"), active=True
        )
        discount.tags.add(tag)
        discount.save()

        item1 = mommy.make(models.StoreItem, pre_tax_price=Decimal("10"), vat_rate=vat_rate)
        item1.tags.add(tag)
        item1.save()

        item2 = mommy.make(models.StoreItem, pre_tax_price=Decimal("10"), vat_rate=vat_rate)
        item2.tags.add(tag)
        item2.save()

        sale = mommy.make(models.Sale)

        sale_item1 = mommy.make(
            models.SaleItem, quantity=3, item=item1, pre_tax_price=item1.pre_tax_price, vat_rate=item1.vat_rate,
            sale=sale
        )

        self.assertEqual(sale.saleitem_set.count(), 1)
        self.assertEqual(Decimal("27"), sale.pre_tax_total_price())
        self.assertEqual(Decimal("32.4"), sale.vat_incl_total_price())

        sale_item1 = models.SaleItem.objects.get(id=sale_item1.id)
        self.assertEqual(sale_item1.discount, discount)
        self.assertEqual(sale_item1.unit_price(), Decimal('9'))
        self.assertEqual(sale_item1.vat_incl_price(), Decimal('10.8'))
        self.assertEqual(sale_item1.vat_price(), Decimal('1.8'))
        self.assertEqual(sale_item1.total_vat_price(), Decimal('5.4'))

        sale_item2 = mommy.make(
            models.SaleItem, quantity=1, item=item1, pre_tax_price=item2.pre_tax_price, vat_rate=item2.vat_rate,
            sale=sale
        )

        self.assertEqual(sale.saleitem_set.count(), 2)
        self.assertEqual(Decimal("37"), sale.pre_tax_total_price())
        self.assertEqual(Decimal("44.4"), sale.vat_incl_total_price())

        sale_item2 = models.SaleItem.objects.get(id=sale_item2.id)
        self.assertEqual(sale_item2.discount, None)
        self.assertEqual(sale_item2.unit_price(), Decimal('10'))
        self.assertEqual(sale_item2.vat_incl_price(), Decimal('12'))
        self.assertEqual(sale_item2.vat_price(), Decimal('2'))
        self.assertEqual(sale_item2.total_vat_price(), Decimal('2'))

        sale_item2.quantity = 3
        sale_item2.save()
        sale_item2 = models.SaleItem.objects.get(id=sale_item2.id)
        self.assertEqual(sale_item2.discount, discount)
        self.assertEqual(sale_item2.unit_price(), Decimal('9'))
        self.assertEqual(sale_item2.vat_incl_price(), Decimal('10.8'))
        self.assertEqual(sale_item2.vat_price(), Decimal('1.8'))
        self.assertEqual(sale_item2.total_vat_price(), Decimal('5.4'))

        sale = models.Sale.objects.get(id=sale.id)
        self.assertEqual(sale.saleitem_set.count(), 2)
        self.assertEqual(Decimal("54"), sale.pre_tax_total_price())
        self.assertEqual(Decimal("64.8"), sale.vat_incl_total_price())


class StoreItemDiscountTest(TestCase):
    """It should return the list of discounts associated with an item"""

    def test_view_discounts(self):
        """if several tags are associated"""

        tag1 = mommy.make(models.StoreItemTag, name="vrac")
        tag2 = mommy.make(models.StoreItemTag, name="promo")
        tag3 = mommy.make(models.StoreItemTag, name="soldes")
        tag4 = mommy.make(models.StoreItemTag, name="super-promos")

        discount = mommy.make(
            models.Discount, name='-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=True
        )
        discount.tags.add(tag1)
        discount.tags.add(tag2)
        discount.save()

        discount2 = mommy.make(
            models.Discount, name='-20% à partir de 3 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
        )
        discount2.tags.add(tag1)
        discount2.save()

        discount3 = mommy.make(
            models.Discount, name='-80% à partir de 3 Kg', rate=Decimal("80"), quantity=Decimal("3"), active=True
        )

        item = mommy.make(models.StoreItem)
        item.tags.add(tag1)
        item.tags.add(tag2)
        item.save()

        self.assertEqual(list(item.discounts), [discount2, discount])

    def test_view_discounts_price_class(self):
        """if several tags are associated"""

        discount = mommy.make(
            models.Discount, name='-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=True
        )

        price_class = mommy.make(models.PriceClass)
        price_class.discounts.add(discount)
        price_class.save()

        item = mommy.make(models.StoreItem, price_class=price_class)

        self.assertEqual(list(item.discounts), [discount])

    def test_view_discounts_tag_and_price_class(self):
        """if tags and price classes are associated"""
        tag1 = mommy.make(models.StoreItemTag, name="vrac")
        tag2 = mommy.make(models.StoreItemTag, name="promo")

        discount = mommy.make(
            models.Discount, name='-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=True
        )
        discount.tags.add(tag1)
        discount.tags.add(tag2)
        discount.save()

        discount2 = mommy.make(
            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
        )
        discount2.tags.add(tag1)
        discount2.save()

        price_class = mommy.make(models.PriceClass)
        price_class.discounts.add(discount)
        price_class.save()

        item = mommy.make(models.StoreItem, price_class=price_class)
        item.tags.add(tag1)
        item.tags.add(tag2)
        item.save()

        discounts = list(item.discounts)
        self.assertEqual(discounts, [discount2, discount])

    def test_view_discounts_tag_and_price_class2(self):
        """if tags and price classes are associated"""
        tag1 = mommy.make(models.StoreItemTag, name="vrac")

        discount = mommy.make(
            models.Discount, name='-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=True
        )
        discount.tags.add(tag1)
        discount.save()

        discount2 = mommy.make(
            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
        )

        price_class = mommy.make(models.PriceClass)
        price_class.discounts.add(discount2)
        price_class.save()

        item = mommy.make(models.StoreItem, price_class=price_class)
        item.tags.add(tag1)
        item.save()

        self.assertEqual(list(item.discounts), [discount2, discount])

    def test_view_price_class_several_discounts(self):
        """if several price classes are associated"""
        discount = mommy.make(
            models.Discount, name='-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=True
        )

        discount2 = mommy.make(
            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
        )

        price_class = mommy.make(models.PriceClass)
        price_class.discounts.add(discount)
        price_class.discounts.add(discount2)
        price_class.save()

        item = mommy.make(models.StoreItem, price_class=price_class)
        self.assertEqual(list(item.discounts), [discount2, discount])

    def test_view_price_class_no_discounts(self):
        """if no price classes are associated"""
        discount = mommy.make(
            models.Discount, name='-20% à partir de 4 Kg', rate=Decimal("20"), quantity=Decimal("4"), active=False
        )

        mommy.make(
            models.Discount, name='-10% à partir de 2 Kg', rate=Decimal("10"), quantity=Decimal("2"), active=True
        )

        price_class = mommy.make(models.PriceClass)
        price_class.discounts.add(discount)
        price_class.save()

        item = mommy.make(models.StoreItem, price_class=price_class)
        self.assertEqual(list(item.discounts), [])

    def test_view_discounts_inactive(self):
        """If only 1 tag"""

        tag1 = mommy.make(models.StoreItemTag, name="vrac")
        tag2 = mommy.make(models.StoreItemTag, name="promo")
        tag3 = mommy.make(models.StoreItemTag, name="soldes")

        discount = mommy.make(
            models.Discount, name='-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=False
        )
        discount.tags.add(tag1)
        discount.tags.add(tag2)
        discount.save()

        discount2 = mommy.make(
            models.Discount, name='-20% à partir de 3 Kg', rate=Decimal("20"), quantity=Decimal("3"), active=True
        )
        discount2.tags.add(tag1)
        discount2.save()

        discount3 = mommy.make(
            models.Discount, name='-80% à partir de 3 Kg', rate=Decimal("80"), quantity=Decimal("3"), active=True
        )

        item = mommy.make(models.StoreItem)
        item.tags.add(tag1)
        item.tags.add(tag2)
        item.save()

        self.assertEqual(list(item.discounts), [discount2])

    def test_view_discounts_none(self):
        """if no tag"""

        tag1 = mommy.make(models.StoreItemTag, name="vrac")
        tag2 = mommy.make(models.StoreItemTag, name="promo")

        discount = mommy.make(
            models.Discount, name='-50% à partir de 4 Kg', rate=Decimal("50"), quantity=Decimal("4"), active=False
        )
        discount.tags.add(tag1)
        discount.tags.add(tag2)
        discount.save()

        item = mommy.make(models.StoreItem)

        self.assertEqual(list(item.discounts), [])


class MailtoActionTest(TestCase):
    """mailto"""

    def setUp(self):
        """before each test"""
        super(MailtoActionTest, self).setUp()
        user = mommy.make(User, is_active=True, is_staff=True, is_superuser=False)
        user.set_password('abc')
        user.save()
        self.client.login(username=user.username, password='abc')

    def test_mailto_to_action_body(self):

        action_type = mommy.make(ActionType, mail_to_subject='', generate_uuid=True, name='Doc')
        mommy.make(models.StoreManagementActionType, action_type=action_type)

        action = mommy.make(Action, type=action_type, subject='Test')

        contact = mommy.make(Contact, email='toto@toto.fr', lastname='Doe', firstname='John')
        action.contacts.add(contact)
        action.save()

        self.assertNotEqual(action.uuid, '')
        url = reverse('store_view_sales_document_public', args=[action.uuid])

        expected_body = "Here is a link to your document: https://example.com{0}".format(url).replace(' ', '%20')

        mommy.make(
            MailtoSettings,
            body_template="Here is a link to your document: {{ doc_url }}",
            action_type=action_type
        )

        url = reverse("crm_mailto_action", args=[action.id])
        self.assertEqual(action.mail_to, url)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body={0}".format(expected_body))

    def test_mailto_to_action_body_no_sale(self):

        action_type = mommy.make(ActionType, mail_to_subject='', generate_uuid=True, name='Doc')

        action = mommy.make(Action, type=action_type, subject='Test')

        contact = mommy.make(Contact, email='toto@toto.fr', lastname='Doe', firstname='John')
        action.contacts.add(contact)
        action.save()

        mommy.make(MailtoSettings, body_template="", action_type=action_type)

        self.assertNotEqual(action.uuid, '')

        url = reverse("crm_mailto_action", args=[action.id])
        self.assertEqual(action.mail_to, url)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=")

    def test_mailto_to_action_body_no_settings(self):

        action_type = mommy.make(ActionType, mail_to_subject='', generate_uuid=True, name='Doc')

        action = mommy.make(Action, type=action_type, subject='Test')

        contact = mommy.make(Contact, email='toto@toto.fr', lastname='Doe', firstname='John')
        action.contacts.add(contact)
        action.save()

        self.assertNotEqual(action.uuid, '')

        url = reverse("crm_mailto_action", args=[action.id])
        self.assertEqual(action.mail_to, url)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=")

    def test_mailto_to_action_body_no_uuid(self):

        action_type = mommy.make(ActionType, mail_to_subject='', generate_uuid=False, name='Doc')
        mommy.make(models.StoreManagementActionType, action_type=action_type)

        action = mommy.make(Action, type=action_type, subject='Test')

        contact = mommy.make(Contact, email='toto@toto.fr', lastname='Doe', firstname='John')
        action.contacts.add(contact)
        action.save()

        self.assertEqual(action.uuid, '')

        url = reverse("crm_mailto_action", args=[action.id])
        self.assertEqual(action.mail_to, url)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=")

    def test_mailto_to_action_subject(self):

        action_type = mommy.make(ActionType, mail_to_subject='Another subject', generate_uuid=True, name='Doc')
        mommy.make(models.StoreManagementActionType, action_type=action_type)

        action = mommy.make(Action, type=action_type, subject='Test')

        contact = mommy.make(Contact, email='toto@toto.fr', lastname='Doe', firstname='John')
        action.contacts.add(contact)
        action.save()

        self.assertNotEqual(action.uuid, '')
        url = reverse('store_view_sales_document_public', args=[action.uuid])

        url = reverse("crm_mailto_action", args=[action.id])
        self.assertEqual(action.mail_to, url)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Another%20subject&body=")


class SaleItemOrderTest(TestCase):
    """It should increment the order correctly"""

    def test_default(self):
        """It should be 1"""
        sale = mommy.make(models.Sale)
        item = mommy.make(models.SaleItem, sale=sale, pre_tax_price="10")
        self.assertEqual(1, item.order_index)

    def test_zero(self):
        """It should be 1"""
        sale = mommy.make(models.Sale)
        item = mommy.make(models.SaleItem, sale=sale, order_index=0, pre_tax_price="10")
        self.assertEqual(1, item.order_index)

    def test_max(self):
        """It should be max + 1"""
        sale = mommy.make(models.Sale)
        mommy.make(models.SaleItem, sale=sale, order_index=10, pre_tax_price="10")
        item = mommy.make(models.SaleItem, sale=sale, pre_tax_price="10")
        self.assertEqual(11, item.order_index)

    def test_value(self):
        """It should be the given value"""
        sale = mommy.make(models.Sale)
        item = mommy.make(models.SaleItem, sale=sale, order_index=5, pre_tax_price="10")
        self.assertEqual(5, item.order_index)

    def test_max_other(self):
        """It should be 1 (max of the sale)"""
        sale = mommy.make(models.Sale)
        other_sale = mommy.make(models.Sale)
        mommy.make(models.SaleItem, sale=other_sale, order_index=10, pre_tax_price="10")
        item = mommy.make(models.SaleItem, sale=sale, pre_tax_price="10")
        self.assertEqual(1, item.order_index)


class SaleReferenceTest(TestCase):
    """It should increment the order correctly"""

    def test_empty(self):
        """It should be empty"""
        action_type = mommy.make(ActionType)
        mommy.make(models.StoreManagementActionType, action_type=action_type)

        action = mommy.make(Action, type=action_type)

        self.assertNotEqual(action.sale, None)
        self.assertEqual(action.sale.get_references_text(), "")

    def test_value(self):
        """It should be empty"""
        action_type = mommy.make(ActionType)
        references_text = "<p>Hello</p>"
        mommy.make(models.StoreManagementActionType, action_type=action_type, references_text=references_text)

        action = mommy.make(Action, type=action_type)

        self.assertNotEqual(action.sale, None)
        self.assertEqual(action.sale.get_references_text(), references_text)


class FreezeActionTest(TestCase):
    """It should increment the order correctly"""

    def test_not_frozen(self):
        """It should be empty"""
        status1 = mommy.make(ActionStatus)
        status2 = mommy.make(ActionStatus)
        status3 = mommy.make(ActionStatus)

        store_action_type = mommy.make(models.StoreManagementActionType)
        store_action_type.default_readonly_status = status3
        store_action_type.readonly_status.add(status2, status3)
        store_action_type.save()

        action = mommy.make(Action, type=store_action_type.action_type, status=status1)
        self.assertEqual(action.frozen, False)

    def test_frozen(self):
        """It should be empty"""
        status1 = mommy.make(ActionStatus)
        status2 = mommy.make(ActionStatus)
        status3 = mommy.make(ActionStatus)

        store_action_type = mommy.make(models.StoreManagementActionType)
        store_action_type.default_readonly_status = status3
        store_action_type.readonly_status.add(status2, status3)
        store_action_type.save()

        action = mommy.make(Action, type=store_action_type.action_type, status=status2)
        self.assertEqual(action.frozen, True)

    def test_refreeze(self):
        """It should be empty"""
        status1 = mommy.make(ActionStatus)
        status2 = mommy.make(ActionStatus)
        status3 = mommy.make(ActionStatus)

        store_action_type = mommy.make(models.StoreManagementActionType)
        store_action_type.default_readonly_status = status3
        store_action_type.readonly_status.add(status2, status3)
        store_action_type.save()

        action = mommy.make(Action, type=store_action_type.action_type, status=status1)
        self.assertEqual(action.frozen, False)

        action.frozen = True
        action.save()
        self.assertEqual(action.frozen, True)
        self.assertEqual(action.status, status1)

        action.status = status2
        action.save()
        self.assertEqual(action.frozen, True)
        self.assertEqual(action.status, status2)




# -*- coding: utf-8 -*-
"""a simple store"""

from __future__ import unicode_literals

from decimal import Decimal

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template
from django.utils.translation import ugettext as _

from coop_cms.utils import dehtml

from balafon.utils import logger
from balafon.Store.settings import get_cart_confirmation_subject


def notify_cart_to_admin(profile, action):
    """send message by email"""
    notification_email = getattr(settings, 'BALAFON_NOTIFICATION_EMAIL', '')
    if notification_email:
        from_email = getattr(settings, 'DEFAULT_FROM_EMAIL')
        subject = _("New cart purchased on store")
        data = {
            'profile': profile,
            'action': action,
            'subject': subject,
        }
        the_template = get_template('Store/cart_notification_email.html')
        html_text = the_template.render(data)
        text = dehtml(html_text)

        email = EmailMultiAlternatives(
            subject,
            text,
            from_email,
            [notification_email],
            headers={'Reply-To': profile.contact.email}
        )
        email.attach_alternative(html_text, "text/html")

        try:
            email.send()
        except Exception:
            logger.exception("notify_cart_to_admin")


def confirm_cart_to_user(profile, action):
    """send message by email"""

    from_email = getattr(settings, 'DEFAULT_FROM_EMAIL')
    subject = get_cart_confirmation_subject(profile, action)

    data = {
        'profile': profile,
        'action': action,
        'subject': subject,
    }

    the_template = get_template('Store/cart_confirmation_email.html')
    html_text = the_template.render(data)
    text = dehtml(html_text)

    email = EmailMultiAlternatives(
        subject,
        text,
        from_email,
        [profile.contact.email],
    )
    email.attach_alternative(html_text, "text/html")

    try:
        email.send()
    except Exception:
        logger.exception("confirm_cart_to_user")


def to_decimal(value):
    """convert to Decimal"""
    return Decimal("{0:.2f}".format(value))


def round_currency(value):
    """round a value with two digits"""
    return to_decimal(round(value, 2))


# -*- coding: utf-8 -*-
"""test we can search by groups"""

from __future__ import unicode_literals

from django.core.urlresolvers import reverse

from coop_cms.tests import BeautifulSoup
from model_mommy import mommy

from balafon.Crm import models
from balafon.Search.tests import BaseTestCase


class GroupSearchTest(BaseTestCase):
    """Vith the search when accessing it by the group"""

    def test_view_search(self):
        """view search"""
        url = reverse('search')
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

    def test_search_anonymous(self):
        """view search not logged"""
        self.client.logout()
        response = self.client.get(reverse('search'))
        self.assertEqual(302, response.status_code)
        # login url without lang prefix
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_search_non_staff(self):
        """view seacrch non-staff"""
        self.user.is_staff = False
        self.user.save()
        response = self.client.get(reverse('search'))
        self.assertEqual(302, response.status_code)
        # login url without lang prefix
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_view_group(self):
        """view the search_group"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group = mommy.make(models.Group, name="my group")

        group.entities.add(entity1)
        group.save()

        url = reverse('search_group', args=[group.id])

        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_group(self):
        """search by group"""

        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group = mommy.make(models.Group, name="my group")
        group.entities.add(entity1)
        group.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_contact_group(self):
        """search contact by group"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group = mommy.make(models.Group, name="my group")
        group.contacts.add(contact1)
        group.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_two_group(self):
        """search two groups"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="my group")
        group1.entities.add(entity1)
        group1.save()

        group2 = mommy.make(models.Group, name="oups")
        group2.entities.add(entity1)
        group2.entities.add(entity2)
        group2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group1.id, "gr0-_-group-_-1": group2.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_two_group_with_contacts(self):
        """search two groups with contacts"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="my group")
        group1.contacts.add(contact1)
        group1.save()

        group2 = mommy.make(models.Group, name="oups")
        group2.contacts.add(contact1)
        group2.contacts.add(contact2)
        group2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group1.id, "gr0-_-group-_-1": group2.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_two_group_mix_entity_contacts(self):
        """search two groups contacts and entities"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="my group")
        group1.contacts.add(contact1)
        group1.save()

        group2 = mommy.make(models.Group, name="oups")
        group2.contacts.add(contact1)
        group2.entities.add(entity2)
        group2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group1.id, "gr0-_-group-_-1": group2.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_two_group_not_in(self):
        """search not member of group"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="my group")
        group1.entities.add(entity1)
        group1.save()

        group2 = mommy.make(models.Group, name="oups")
        group2.entities.add(entity1)
        group2.save()

        url = reverse('search')

        data = {"gr0-_-not_in_group-_-0": group1.id, "gr0-_-not_in_group-_-1": group2.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, entity1.name)
        self.assertNotContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

        self.assertContains(response, entity2.name)
        self.assertContains(response, contact2.lastname)

    def test_search_two_group_in_and_not_in(self):
        """search two groups : members and not members"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="my group")
        group1.entities.add(entity1)
        group1.entities.add(entity2)
        group1.save()

        group2 = mommy.make(models.Group, name="oups")
        group2.entities.add(entity1)
        group2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group1.id, "gr0-_-not_in_group-_-1": group2.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, entity1.name)
        self.assertNotContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

        self.assertContains(response, entity2.name)
        self.assertContains(response, contact2.lastname)

    def test_search_contacts_two_group_in_and_not_in(self):
        """search two groups on contacts: members and not members"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="my group")
        group1.contacts.add(contact1)
        group1.contacts.add(contact2)
        group1.save()

        group2 = mommy.make(models.Group, name="oups")
        group2.contacts.add(contact1)
        group2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group1.id, "gr0-_-not_in_group-_-1": group2.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, entity1.name)
        self.assertNotContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

        self.assertContains(response, entity2.name)
        self.assertContains(response, contact2.lastname)

    def test_search_contacts_entities_two_group_in_and_not_in(self):
        """search two groups on entities: members and not members"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="my group")
        group1.entities.add(entity1)
        group1.entities.add(entity2)
        group1.save()

        group2 = mommy.make(models.Group, name="oups")
        group2.contacts.add(contact1)
        group2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group1.id, "gr0-_-not_in_group-_-1": group2.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertNotContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertContains(response, entity2.name)
        self.assertContains(response, contact2.lastname)

    def test_search_groups_absurde(self):
        """search in_group and not_in_group on same group"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="my group")
        group1.entities.add(entity1)
        group1.entities.add(entity2)
        group1.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group1.id, "gr0-_-not_in_group-_-1": group1.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, entity1.name)
        self.assertNotContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

    def test_search_groups_absurde2(self):
        """search in_group and not_in_group on same group: other order"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="my group")
        group1.entities.add(entity1)
        group1.entities.add(entity2)
        group1.save()

        url = reverse('search')

        data = {"gr0-_-not_in_group-_-0": group1.id, "gr0-_-group-_-1": group1.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, entity1.name)
        self.assertNotContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

    def test_search_contact_of_group(self):
        """search contacts of group"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="The big Org")
        contact4 = mommy.make(models.Contact, entity=entity3, lastname="ABCABC", main_contact=True, has_left=False)

        group = mommy.make(models.Group, name="my group")
        group.entities.add(entity1)
        group.save()

        url = reverse('search')

        data = {"gr0-_-contact_name-_-0": 'ABC', 'gr0-_-group-_-1': group.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact4.lastname)


class MultiGroupSearchTest(BaseTestCase):
    """search on multiple groups form"""

    def test_search_all_groups_only_1(self):
        """all groups: members of 1 only"""
        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="#Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="#Big corp")
        contact3 = mommy.make(models.Contact, entity=entity3, lastname="XDER", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="#group1")
        group1.entities.add(entity1)
        group1.save()

        url = reverse('search')
        groups = (group1.id, )
        data = {"gr0-_-all_groups-_-0": ['{0}'.format(x) for x in groups]}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1a.lastname)
        self.assertContains(response, contact1b.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact3.lastname)

    def test_search_all_groups(self):
        """all groups: members of all"""
        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="#Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="#Big corp")
        contact3 = mommy.make(models.Contact, entity=entity3, lastname="XDER", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="#group1")
        group1.entities.add(entity1)
        group1.save()

        group2 = mommy.make(models.Group, name="#group2")
        group2.entities.add(entity1)
        group2.entities.add(entity2)
        group2.save()

        url = reverse('search')
        groups = (group1.id, group2.id)
        data = {"gr0-_-all_groups-_-0": ['{0}'.format(x) for x in groups]}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1a.lastname)
        self.assertContains(response, contact1b.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact3.lastname)

    def test_search_any_groups(self):
        """search member any groups"""
        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="#Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="#Big corp")
        contact3 = mommy.make(models.Contact, entity=entity3, lastname="XDER", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="#group1")
        group1.entities.add(entity1)
        group1.save()

        group2 = mommy.make(models.Group, name="#group2")
        group2.entities.add(entity1)
        group2.entities.add(entity2)
        group2.save()

        url = reverse('search')
        groups = (group1.id, group2.id)
        data = {"gr0-_-any_groups-_-0": ['{0}'.format(x) for x in groups]}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1a.lastname)
        self.assertContains(response, contact1b.lastname)

        self.assertContains(response, entity2.name)
        self.assertContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact3.lastname)

    def test_search_none_groups(self):
        """search contact who are not member of given groups"""
        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="#Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="#Big corp")
        contact3 = mommy.make(models.Contact, entity=entity3, lastname="XDER", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="#group1")
        group1.entities.add(entity1)
        group1.save()

        group2 = mommy.make(models.Group, name="#group2")
        group2.entities.add(entity1)
        group2.entities.add(entity2)
        group2.save()

        url = reverse('search')
        groups = (group1.id, group2.id)
        data = {"gr0-_-none_groups-_-0": ['{0}'.format(x) for x in groups]}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, entity1.name)
        self.assertNotContains(response, contact1a.lastname)
        self.assertNotContains(response, contact1b.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

        self.assertContains(response, entity3.name)
        self.assertContains(response, contact3.lastname)

    def test_search_all_groups_contact(self):
        """search member of all groups: contacts"""
        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="#Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="#Big corp")
        contact3 = mommy.make(models.Contact, entity=entity3, lastname="XDER", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="#group1")
        group1.contacts.add(contact1a)
        group1.save()

        group2 = mommy.make(models.Group, name="#group2")
        group2.contacts.add(contact1a)
        group2.entities.add(entity2)
        group2.save()

        url = reverse('search')
        groups = (group1.id, group2.id)
        data = {"gr0-_-all_groups-_-0": ['{0}'.format(x) for x in groups]}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1a.lastname)
        self.assertNotContains(response, contact1b.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact3.lastname)

    def test_search_any_groups_contact(self):
        """search member of any groups: contacts"""
        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="#Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="#Big corp")
        contact3 = mommy.make(models.Contact, entity=entity3, lastname="XDER", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="#group1")
        group1.contacts.add(contact1a)
        group1.save()

        group2 = mommy.make(models.Group, name="#group2")
        group2.contacts.add(contact1a)
        group2.entities.add(entity2)
        group2.save()

        url = reverse('search')
        groups = (group1.id, group2.id)
        data = {"gr0-_-any_groups-_-0": ['{0}'.format(x) for x in groups]}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1a.lastname)
        self.assertNotContains(response, contact1b.lastname)

        self.assertContains(response, entity2.name)
        self.assertContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact3.lastname)

    def test_search_none_groups_contact(self):
        """search not member of any of the groups: contacts"""
        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="#Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="#Big corp")
        contact3 = mommy.make(models.Contact, entity=entity3, lastname="XDER", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="#group1")
        group1.contacts.add(contact1a)
        group1.save()

        group2 = mommy.make(models.Group, name="#group2")
        group2.contacts.add(contact1a)
        group2.entities.add(entity2)
        group2.save()

        url = reverse('search')
        groups = (group1.id, group2.id)
        data = {"gr0-_-none_groups-_-0": ['{0}'.format(x) for x in groups]}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertNotContains(response, contact1a.lastname)
        self.assertContains(response, contact1b.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

        self.assertContains(response, entity3.name)
        self.assertContains(response, contact3.lastname)

    def test_search_all_groups_no_choice(self):
        """search all_groups: no value set"""
        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        url = reverse('search')
        data = {"gr0-_-all_groups-_-0": ''}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertNotEqual([], soup.select('.field-error'))

        self.assertNotContains(response, entity1.name)
        self.assertNotContains(response, contact1a.lastname)
        self.assertNotContains(response, contact1b.lastname)

    def test_search_any_groups_no_choice(self):
        """search any_groups: no value set"""

        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        url = reverse('search')
        data = {"gr0-_-any_groups-_-0": ''}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertNotEqual([], soup.select('.field-error'))

        self.assertNotContains(response, entity1.name)
        self.assertNotContains(response, contact1a.lastname)
        self.assertNotContains(response, contact1b.lastname)

    def test_search_any_groups_invalid_choice(self):
        """search any_groups: invalid value"""

        url = reverse('search')
        data = {"gr0-_-any_groups-_-0": 'blabla'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertNotEqual([], soup.select('.field-error'))

    def test_search_none_groups_no_choice(self):
        """search none_groups: no value set"""

        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        url = reverse('search')
        data = {"gr0-_-none_groups-_-0": ''}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertNotEqual([], soup.select('.field-error'))

        self.assertNotContains(response, entity1.name)
        self.assertNotContains(response, contact1a.lastname)
        self.assertNotContains(response, contact1b.lastname)

    def test_search_groups_combine_all_none(self):
        """search combining all and none"""
        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="#Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="#Big corp")
        contact3 = mommy.make(models.Contact, entity=entity3, lastname="XDER", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="#group1")
        group1.contacts.add(contact1a)
        group1.save()

        group2 = mommy.make(models.Group, name="#group2")
        group2.contacts.add(contact1a)
        group2.entities.add(entity2)
        group2.save()

        groups = [group1.id, group2.id]

        response = self.client.post(
            reverse('search'),
            data={
                "gr0-_-all_groups-_-0": ['{0}'.format(x) for x in groups],
                "gr0-_-none_groups-_-1": ['{0}'.format(x) for x in groups],
            }
        )
        self.assertEqual(200, response.status_code)

        self.assertEqual([], BeautifulSoup(response.content).select('.field-error'))

        self.assertNotContains(response, entity1.name)
        self.assertNotContains(response, contact1a.lastname)
        self.assertNotContains(response, contact1b.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact3.lastname)

    def test_search_groups_combine_all_any(self):
        """search combining all and any"""
        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="#Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="#Big corp")
        contact3 = mommy.make(models.Contact, entity=entity3, lastname="XDER", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="#group1")
        group1.contacts.add(contact1a)
        group1.save()

        group2 = mommy.make(models.Group, name="#group2")
        group2.contacts.add(contact1a)
        group2.entities.add(entity2)
        group2.save()

        groups = [group1.id, group2.id]

        response = self.client.post(
            reverse('search'),
            data={
                "gr0-_-all_groups-_-0": ['{0}'.format(x) for x in groups],
                "gr0-_-any_groups-_-1": ['{0}'.format(x) for x in groups],
            }
        )
        self.assertEqual(200, response.status_code)

        self.assertEqual([], BeautifulSoup(response.content).select('.field-error'))

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1a.lastname)
        self.assertNotContains(response, contact1b.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact3.lastname)

    def test_search_groups_union_all_any(self):
        """search union of all and any"""
        entity1 = mommy.make(models.Entity, name="#Tiny corp")
        contact1a = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="#Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="#Big corp")
        contact3 = mommy.make(models.Contact, entity=entity3, lastname="XDER", main_contact=True, has_left=False)

        group1 = mommy.make(models.Group, name="#group1")
        group1.contacts.add(contact1a)
        group1.save()

        group2 = mommy.make(models.Group, name="#group2")
        group2.contacts.add(contact1a)
        group2.entities.add(entity2)
        group2.save()

        groups = [group1.id, group2.id]

        response = self.client.post(
            reverse('search'),
            data={
                "gr0-_-all_groups-_-0": ['{0}'.format(x) for x in groups],
                "gr1-_-any_groups-_-0": ['{0}'.format(x) for x in groups],
            }
        )
        self.assertEqual(200, response.status_code)

        self.assertEqual([], BeautifulSoup(response.content).select('.field-error'))

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1a.lastname)
        self.assertNotContains(response, contact1b.lastname)

        self.assertContains(response, entity2.name)
        self.assertContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact3.lastname)


class ContactWithEmailInGroupSearchForm(BaseTestCase):
    """search on multiple groups form"""

    def test_search_all_groups_only_1(self):
        """all groups: members of 1 only"""
        entity1 = mommy.make(models.Entity, name=u"#Tiny corp")
        contact1a = mommy.make(
            models.Contact, entity=entity1, lastname=u"ABCD", main_contact=True, has_left=False, email="test1@abcd.fr"
        )
        contact1b = mommy.make(
            models.Contact, entity=entity1, lastname=u"IJKL", main_contact=True, has_left=False, email="test2@abcd.fr"
        )
        contact1c = mommy.make(
            models.Contact, entity=entity1, lastname=u"MNOP", main_contact=True, has_left=False, email="test1@abcd.fr"
        )
        contact1d = mommy.make(
            models.Contact, entity=entity1, lastname=u"QRST", main_contact=True, has_left=False, email="test3@abcd.fr"
        )

        entity2 = mommy.make(models.Entity, name=u"#Other corp")
        contact2a = mommy.make(
            models.Contact, entity=entity2, lastname=u"UVWX", main_contact=True, has_left=False, email="test1@abcd.fr"
        )
        contact2b = mommy.make(
            models.Contact, entity=entity2, lastname=u"YZAB", main_contact=True, has_left=False, email="test3@abcd.fr"
        )

        group1 = mommy.make(models.Group, name=u"#group1")
        group1.entities.add(entity1)
        group1.save()

        group2 = mommy.make(models.Group, name=u"#group2")
        group2.contacts.add(contact2a)
        group2.contacts.add(contact1b)
        group2.save()

        url = reverse('search')
        groups = (group1.id, )
        data = {'gr0-_-all_groups-_-0': [str(x) for x in groups], "gr0-_-email_in_group-_-1": group2.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1a.lastname)
        self.assertContains(response, contact1b.lastname)
        self.assertContains(response, contact1c.lastname)
        self.assertNotContains(response, contact1d.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2a.lastname)
        self.assertNotContains(response, contact2b.lastname)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-06 13:44
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0021_auto_20180406_1148'),
    ]

    operations = [
        migrations.AddField(
            model_name='actiontype',
            name='label',
            field=models.CharField(blank=True, default='', help_text='If set, replace name on commercial document', max_length=100, verbose_name='label'),
        )
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Search',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name', models.CharField(max_length=100, verbose_name='name')),
            ],
            options={
                'verbose_name': 'search',
                'verbose_name_plural': 'searchs',
            },
        ),
        migrations.CreateModel(
            name='SearchField',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('field', models.CharField(max_length=100, verbose_name='field')),
                ('value', models.CharField(max_length=200, verbose_name='value')),
                ('is_list', models.BooleanField(default=False)),
                ('count', models.IntegerField(default=0)),
            ],
            options={
                'verbose_name': 'search field',
                'verbose_name_plural': 'search fied',
            },
        ),
        migrations.CreateModel(
            name='SearchGroup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('search', models.ForeignKey(verbose_name='search', to='Search.Search')),
            ],
            options={
                'verbose_name': 'search group',
                'verbose_name_plural': 'search groups',
            },
        ),
        migrations.AddField(
            model_name='searchfield',
            name='search_group',
            field=models.ForeignKey(verbose_name='search group', to='Search.SearchGroup'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0012_auto_20160601_1128'),
        ('Store', '0005_auto_20160601_1128'),
    ]

    operations = [
        migrations.AddField(
            model_name='storeitem',
            name='only_for_groups',
            field=models.ManyToManyField(help_text='If defined, only members of these groups will be able to see the item', to='Crm.Group', verbose_name='only for groups', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""search forms : this forms are included in main search form"""

from __future__ import unicode_literals

from django.db.models import Q
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _

import floppyforms.__future__ as forms

from balafon.Emailing import models
from balafon.Search.forms import SearchFieldForm


class EmailingSearchForm(SearchFieldForm):
    """Base class for emailing searches"""
    allowed_status = []

    def __init__(self, *args, **kwargs):
        super(EmailingSearchForm, self).__init__(*args, **kwargs)
        
        queryset = models.Emailing.objects.all()
        if self.allowed_status:
            queryset = queryset.filter(status__in=self.allowed_status)
        queryset = queryset.order_by("-created", "-id")

        field = forms.ModelChoiceField(queryset, label=self.label, **kwargs)
        self._add_field(field)
        
    def _get_emailing(self):
        """get list of emailings"""
        try:
            return models.Emailing.objects.get(id=self.value)
        except models.Emailing.DoesNotExist:
            raise ValidationError(_("Unknown emailing"))


class EmailingSentSearchForm(EmailingSearchForm):
    """Emailing sent to"""
    name = 'emailing_sent'
    label = _('Emailing sent')
    allowed_status = [models.Emailing.STATUS_SENDING, models.Emailing.STATUS_SENT]
    
    def get_lookup(self):
        """get all contacts who received the emailing"""
        emailing = self._get_emailing()
        return {"emailing_received": emailing.id}
        

class EmailingOpenedSearchForm(EmailingSearchForm):
    """Emailing opened"""
    name = 'emailing_opened'
    label = _('Emailing opened')
    allowed_status = [models.Emailing.STATUS_SENDING, models.Emailing.STATUS_SENT]
    
    def get_lookup(self):
        """get all contacts who opened the emailing"""
        emailing = self._get_emailing()
        return {"emailing_opened": emailing.id}


class EmailingSendToSearchForm(EmailingSearchForm):
    """Emailing send to"""
    name = 'emailing_send'
    label = _('Emailing send to')
    allowed_status = [models.Emailing.STATUS_SENDING, models.Emailing.STATUS_EDITING, models.Emailing.STATUS_SCHEDULED]

    def get_lookup(self):
        """get all contacts for whom we send the emailing"""
        emailing = self._get_emailing()
        return {"emailing_to_be_received": emailing.id}


class EmailingBounceSearchForm(EmailingSearchForm):
    """Emailing bounce"""
    name = 'emailing_bounce'
    label = _('Emailing bounce')
    allowed_status = [models.Emailing.STATUS_SENDING, models.Emailing.STATUS_SENT]

    def get_lookup(self):
        """get all contacts with a bounce for this emailing"""
        emailing = self._get_emailing()
        q_objects = Q(emailing_hard_bounce=emailing.id)
        q_objects |= Q(emailing_soft_bounce=emailing.id)
        q_objects |= Q(emailing_spam=emailing.id)
        q_objects |= Q(emailing_unsub=emailing.id)
        q_objects |= Q(emailing_rejected=emailing.id)
        return q_objects


class EmailingContactsSearchForm(EmailingSearchForm):
    """Emailing send to"""
    name = 'emailing_contacts'
    label = _('Emailing contacts')

    def get_lookup(self):
        """get all contacts for this emailing"""
        emailing = self._get_emailing()
        return Q(emailing_received=emailing.id) | Q(emailing_to_be_received=emailing.id)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function

from django.core.management.base import BaseCommand

from balafon.Crm import models
from balafon.Crm.settings import ALLOW_SINGLE_CONTACT


class Command(BaseCommand):
    help = "add a new credit of emailings"
    use_argparse = False

    def handle(self, *args, **options):
        verbose = options.get('verbosity', 0)

        if not ALLOW_SINGLE_CONTACT:
            print("this command is disabled if BALAFON_ALLOW_SINGLE_CONTACT is not set")
            return
        
        for entity in models.Entity.objects.filter(is_single_contact=True):
            entity.name = "{0.lastname} {0.firstname}".format(entity.default_contact).strip().lower()
            entity.save()
        
            if verbose:
                print(entity.name)
        print(models.Entity.objects.filter(is_single_contact=True).count(), "entities have been renamed")



# -*- coding: utf-8 -*-
"""forms than can be included as part of the main search form"""

from __future__ import unicode_literals

from datetime import date, timedelta

from django.db.models import Q, Count
from django.contrib.auth.models import User
from django.utils.translation import ugettext as _

import floppyforms.__future__ as forms

from balafon.Crm import models
from balafon.Crm.settings import get_language_choices
from balafon.Crm.utils import (
    get_default_country, sort_by_name_callback, sort_by_entity_callback, sort_by_contact_callback
)
from balafon.Crm.widgets import CityNoCountryAutoComplete, GroupAutoComplete
from balafon.Search.forms import SearchFieldForm, TwoDatesForm, YesNoSearchFieldForm


class EntityNameSearchForm(SearchFieldForm):
    """by entity name"""
    name = 'entity_name'
    label = _('Entity name')
    
    def __init__(self, *args, **kwargs):
        super(EntityNameSearchForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of the name of the searched entities')})
        )
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return {'entity__name__icontains': self.value}
    

class EntityDescriptionForm(SearchFieldForm):
    """by entity description"""

    name = 'entity_description'
    label = _('Entity description')
    
    def __init__(self, *args, **kwargs):
        super(EntityDescriptionForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(
                attrs={'placeholder': _('Enter a part of the description of the searched entities')}
            )
        )
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return {'entity__description__icontains': self.value}


class EntityNotesForm(SearchFieldForm):
    """by entity notes"""

    name = 'entity_notes'
    label = _('Entity notes')
    
    def __init__(self, *args, **kwargs):
        super(EntityNotesForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of the notes of the searched entities')})
        )
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return {'entity__notes__icontains': self.value}


class EntityNameStartsWithSearchForm(SearchFieldForm):
    """by entity name starts with"""

    name = 'entity_name_sw'
    label = _('Entity name starts with')
    
    def __init__(self, *args, **kwargs):
        super(EntityNameStartsWithSearchForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(
                attrs={'placeholder': _('Enter the beginning of the name of the searched entities')}
            )
        )
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return {'entity__name__istartswith': self.value}


class AddressSearchForm(SearchFieldForm):
    """by address contains"""

    name = 'address'
    label = _('Contact or entity at this address')

    def __init__(self, *args, **kwargs):
        super(AddressSearchForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(
                attrs={'placeholder': _('Enter text contained in an address')}
            )
        )
        self._add_field(field)

    def get_lookup(self):
        """lookup"""
        value = self.value
        return (
            Q(address__icontains=value) | Q(address2__icontains=value) | Q(address3__icontains=value) |
            Q(entity__address__icontains=value) | Q(entity__address2__icontains=value) |
            Q(entity__address3__icontains=value)
        )


class HasEntityForm(YesNoSearchFieldForm):
    """contacts member of an entity"""

    name = 'has_entity'
    label = _('Has entity?')
        
    def get_lookup(self):
        """lookup"""
        no_entity = Q(entity__is_single_contact=True)
        if self.is_yes():
            return ~no_entity
        else:
            return no_entity


class BaseCitySearchForm(SearchFieldForm):
    """base class for search by city"""
    def __init__(self, *args, **kwargs):
        super(BaseCitySearchForm, self).__init__(*args, **kwargs)
        queryset = models.City.objects.order_by('name')
        field = forms.ModelChoiceField(
            queryset,
            label=self.label,
            widget=CityNoCountryAutoComplete(attrs={'placeholder': _('Enter a city'), 'size': '80'})
        )
        self._add_field(field)


class CitySearchForm(BaseCitySearchForm):
    """by city"""
    name = 'city'
    label = _('City')
    
    def get_lookup(self):
        """lookup"""
        return Q(city__id=self.value) | (Q(city__isnull=True) & Q(entity__city__id=self.value))
    

class EntityCitySearchForm(BaseCitySearchForm):
    """by city of the entity"""
    name = 'entity_city'
    label = _('Entity city')

    def get_lookup(self):
        """lookup"""
        return Q(entity__city__id=self.value)


class BaseZipCodeSearchForm(SearchFieldForm):
    """base class for search by zipcode"""
    def __init__(self, *args, **kwargs):
        super(BaseZipCodeSearchForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(attrs={'placeholder': _('Enter the beginning of the zip code')})
        )
        self._add_field(field)
    

class ZipCodeSearchForm(BaseZipCodeSearchForm):
    """by zip code"""
    name = 'zip_code'
    label = _('zip code')
    
    def get_lookup(self):
        """lookup"""
        return Q(zip_code__istartswith=self.value) | (Q(zip_code="") & Q(entity__zip_code__istartswith=self.value))


class EntityZipCodeSearchForm(BaseZipCodeSearchForm):
    """by zip code of the entity"""
    name = 'entity_zip_code'
    label = _('Entity zip code')
            
    def get_lookup(self):
        """lookup"""
        return Q(entity__zip_code__istartswith=self.value)
        

class ZoneSearchForm(SearchFieldForm):
    """By zone"""
    multi_values = True

    def __init__(self, *args, **kwargs):
        super(ZoneSearchForm, self).__init__(*args, **kwargs)
        type_name = self.name.replace("entity_", "")
        queryset = models.Zone.objects.filter(type__type=type_name).order_by('code', 'name')
        kwargs = kwargs or {}
        kwargs.setdefault('required', True)
        widget = self._get_widget()
        if widget:
            kwargs['widget'] = widget
        field = forms.MultipleChoiceField(
            choices=[(x.id, '{0}'.format(x)) for x in queryset.all()],
            label=self.label,
            **kwargs
        )
        self._add_field(field)
        
    def get_values(self):
        """get possible values"""
        if type(self.value) == list:
            values = self.value
        else:
            values = [self.value]
        return values
    
    def _get_widget(self):
        """get widget"""
        return forms.SelectMultiple(attrs={
            'class': "chosen-select",
            'data-placeholder': self.label,
            'style': "width: 100%", 
        })
    
    def get_queryset(self, queryset):
        """queryset"""
        group_queryset = Q(id=0)
        for zone_id in self.get_values():
            self.value = zone_id
            group_queryset = group_queryset | self.get_lookup()
        return queryset.filter(group_queryset)


class DepartmentSearchForm(ZoneSearchForm):
    """by departement"""
    name = 'department'
    label = _('Departments')
        
    def get_lookup(self):
        """lookup"""
        qobj1 = Q(city__parent__id=self.value) & Q(city__parent__type__type="department")

        qobj2 = Q(
            city__isnull=True
        ) & Q(
            entity__city__parent__id=self.value
        ) & Q(
            entity__city__parent__type__type="department"
        )
        
        return qobj1 | qobj2


class EntityDepartmentSearchForm(ZoneSearchForm):
    """by departement of the entity"""
    name = 'entity_department'
    label = _('Entity Departments')
        
    def get_queryset(self, queryset):
        """queryset"""
        queryset = super(EntityDepartmentSearchForm, self).get_queryset(queryset)
        return queryset.filter(entity__city__parent__type__type="department")

    def get_lookup(self):
        """lookup"""
        return Q(entity__city__parent__id=self.value)
        

class RegionSearchForm(ZoneSearchForm):
    """by region"""
    name = 'region'
    label = _('Regions')
        
    def get_lookup(self):
        """lookup"""
        qobj1 = Q(city__parent__parent__id=self.value) & Q(city__parent__parent__type__type="region")

        qobj2 = Q(
            city__isnull=True
        ) & Q(
            entity__city__parent__parent__id=self.value
        ) & Q(
            entity__city__parent__parent__type__type="region"
        )

        return qobj1 | qobj2


class EntityRegionSearchForm(ZoneSearchForm):
    """by region of teh entity"""
    name = 'entity_region'
    label = _('Entity Regions')
    
    def get_queryset(self, queryset):
        """queryset"""
        queryset = super(EntityRegionSearchForm, self).get_queryset(queryset)
        return queryset.filter(entity__city__parent__type__type="department")

    def get_lookup(self):
        """queryset"""
        return Q(entity__city__parent__parent__id=self.value)


class LargeRegionSearchForm(ZoneSearchForm):
    """by region"""
    name = 'large_region'
    label = _('Large Regions')

    def get_lookup(self):
        """lookup"""
        qobj1 = Q(city__parent__parent__groups__id=self.value)

        qobj2 = Q(
            city__isnull=True
        ) & Q(
            entity__city__parent__parent__groups__id=self.value
        )

        return qobj1 | qobj2


class EntityLargeRegionSearchForm(ZoneSearchForm):
    """by region of teh entity"""
    name = 'entity_large_region'
    label = _('Entity Large Regions')

    def get_lookup(self):
        """queryset"""
        return Q(entity__city__parent__parent__groups__id=self.value)


class CountrySearchForm(ZoneSearchForm):
    """by country"""
    name = 'country'
    label = _('Countries')
        
    def get_lookup(self):
        """lookup"""
        default_country = get_default_country()
        if int(self.value) == default_country.id:
            return Q(
                city__parent__type__type='department'
            ) | (Q(
                city__isnull=True
            ) & Q(entity__city__parent__type__type='department'))
        else:
            return Q(city__parent__id=self.value) | (Q(city__isnull=True) & Q(entity__city__parent__id=self.value))


class EntityCountrySearchForm(ZoneSearchForm):
    """by country of the entity"""
    name = 'entity_country'
    label = _('Entity Countries')
        
    def get_lookup(self):
        """lookup"""
        default_country = get_default_country()
        
        if int(self.value) == default_country.id:
            return Q(entity__city__parent__type__type='department')
        else:
            return Q(entity__city__parent__id=self.value, entity__city__parent__type__type="country")


class ZoneGroupSearchForm(ZoneSearchForm):
    """by zone group"""
    name = 'zone_group'
    label = _('Zone Groups')
        
    def get_lookup(self):
        """lookup"""
        qobj1 = Q(city__groups__id=self.value)
        qobj2 = Q(city__isnull=True) & Q(entity__city__groups__id=self.value)
        return qobj1 | qobj2
    

class EntityZoneGroupSearchForm(ZoneSearchForm):
    """by zone group of entity"""
    name = 'entity_zone_group'
    label = _('Entity Zone Groups')
        
    def get_lookup(self):
        """lookup"""
        return Q(entity__city__groups__id=self.value)


class HasCityAndZipcodeForm(YesNoSearchFieldForm):
    """by has city and address"""
    name = 'has_city_and_zip'
    label = _('Has city and zip code?')
        
    def get_lookup(self):
        """lookup"""
        contact_has_address = ~Q(zip_code='') & Q(city__isnull=False)
        entity_has_address = ~Q(entity__zip_code='') & Q(entity__city__isnull=False)
        has_address = contact_has_address | entity_has_address
        if self.is_yes():
            return has_address
        else:
            return ~has_address


class HasAddresseForm(YesNoSearchFieldForm):
    """by has city and address"""
    name = 'has_address'
    label = _('Has address?')

    def get_lookup(self):
        """lookup"""
        contact_has_address = ~Q(zip_code='') & Q(city__isnull=False) & ~Q(address='')
        entity_has_address = ~Q(entity__zip_code='') & Q(entity__city__isnull=False) & ~Q(entity__address='')
        has_address = contact_has_address | entity_has_address
        if self.is_yes():
            return has_address
        else:
            return ~has_address
            

class ActionInProgressForm(YesNoSearchFieldForm):
    """by action in progress"""
    name = 'action'
    label = _('Action in progress')
    is_action_form = True

    def get_queryset(self, queryset):
        """queryset"""
        q_objs = Q(done=False)
        if self.is_yes():
            return queryset.filter(q_objs)
        else:
            return queryset.exclude(q_objs)


class HasAction(YesNoSearchFieldForm):
    """Has an action"""
    name = 'has_action'
    label = _('Has actions')
        
    def get_queryset(self, queryset):
        """queryset"""
        queryset = queryset.annotate(num_actions=Count('action'), num_entity_actions=Count('entity__action'))
        if self.is_yes():
            return queryset.filter(Q(num_actions__gt=0) | Q(num_entity_actions__gt=0))
        else:
            return queryset.filter().filter(Q(num_actions=0) & Q(num_entity_actions=0))


class ActionByDoneDate(TwoDatesForm):
    """by action done between two dates"""
    name = 'action_by_done_date'
    label = _('Action by done date')
    is_action_form = True
        
    def get_lookup(self):
        """lookup"""
        datetime1, datetime2 = self._get_datetimes()
        return Q(done_date__gte=datetime1) & Q(done_date__lte=datetime2)


class ActionByPlannedDate(TwoDatesForm):
    """by action planned between two dates"""
    name = 'action_by_planned_date'
    label = _('Action by planned date')
    is_action_form = True
    
    def get_lookup(self):
        """lookup"""
        datetime1, datetime2 = self._get_datetimes()
        
        start_after_end_before = Q(
            end_datetime__isnull=True
        ) & Q(
            planned_date__gte=datetime1
        ) & Q(
            planned_date__lte=datetime2
        )

        start_before_end_after = Q(
            end_datetime__isnull=False
        ) & Q(
            planned_date__lte=datetime2
        ) & Q(
            end_datetime__gte=datetime1
        )

        return start_after_end_before | start_before_end_after


class ActionByStartDate(TwoDatesForm):
    """By action started between two dates"""
    name = 'action_by_start_date'
    label = _('Action by start date')
    is_action_form = True
    
    def get_lookup(self):
        """lookup"""
        datetime1, datetime2 = self._get_datetimes()
        return Q(planned_date__gte=datetime1) & Q(planned_date__lte=datetime2)


class ActionByUser(SearchFieldForm):
    """by user in charge of an action"""
    name = 'action_by_user'
    label = _('Action by user')
    is_action_form = True
    
    def __init__(self, *args, **kwargs):
        super(ActionByUser, self).__init__(*args, **kwargs)
        choices = [(team_member.id, team_member.name) for team_member in models.TeamMember.objects.all()]
        field = forms.ChoiceField(choices=choices, label=self.label)
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return Q(in_charge=self.value)


class ActionGteAmount(SearchFieldForm):
    """by action with amount greater than a value"""
    name = 'action_gte_amount'
    label = _('Action with amount greater or equal to')
    is_action_form = True
    
    def __init__(self, *args, **kwargs):
        super(ActionGteAmount, self).__init__(*args, **kwargs)
        field = forms.IntegerField(label=self.label)
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return Q(amount__gte=self.value)


class ActionLtAmount(SearchFieldForm):
    """by action with amount less than a value"""
    name = 'action_lt_amount'
    label = _('Action with amount less than')
    is_action_form = True
    
    def __init__(self, *args, **kwargs):
        super(ActionLtAmount, self).__init__(*args, **kwargs)
        field = forms.IntegerField(label=self.label)
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return Q(amount__lt=self.value)


class ActionStatus(SearchFieldForm):
    """by action status"""
    name = 'action_status'
    label = _('Contacts with action of status')
    is_action_form = True
    
    def __init__(self, *args, **kwargs):
        super(ActionStatus, self).__init__(*args, **kwargs)
        queryset = models.ActionStatus.objects.all()
        field = forms.ModelChoiceField(queryset, label=self.label)
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return Q(status=self.value)


class ActionWithoutStatus(SearchFieldForm):
    """by action status"""
    name = 'action_without_status'
    label = _('Contacts without action of status')
    is_action_form = True

    def __init__(self, *args, **kwargs):
        super(ActionWithoutStatus, self).__init__(*args, **kwargs)
        queryset = models.ActionStatus.objects.all()
        field = forms.ModelChoiceField(queryset, label=self.label)
        self._add_field(field)

    def get_lookup(self):
        """lookup"""
        return ~Q(status=self.value)


class TypeSearchForm(SearchFieldForm):
    """by entity type"""
    name = 'type'
    label = _('Entity type')
    
    def __init__(self, *args, **kwargs):
        super(TypeSearchForm, self).__init__(*args, **kwargs)
        queryset = models.EntityType.objects.all()
        field = forms.ModelChoiceField(queryset, label=self.label)
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return {'entity__type__id': self.value}


class GroupSearchForm(SearchFieldForm):
    """by group"""
    name = 'group'
    label = _('Group')
    
    def _get_widget(self):
        """customize widget: autocomplete"""
        return GroupAutoComplete(attrs={
            'placeholder': _('Enter part of the group name'), 'size': '80',
        })

    def __init__(self, *args, **kwargs):
        super(GroupSearchForm, self).__init__(*args, **kwargs)
        queryset = models.Group.objects.all()
        try:
            queryset = queryset.extra(select={'lower_name': 'lower(name)'}).order_by('lower_name')
        except Exception:  # pylint: disable=broad-except
            queryset = queryset.order_by('name')
        kwargs = {}
        widget = self._get_widget()
        if widget:
            kwargs['widget'] = widget
        field = forms.ModelChoiceField(queryset, label=self.label, **kwargs)
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return Q(entity__group__id=self.value) | Q(group__id=self.value)


class GroupSearchFormDropdownWidget(GroupSearchForm):
    """Search by group: dropdown widget"""
    name = 'group_dropdown'
    label = _('Group (dropdown list)')
    
    def _get_widget(self):
        """dropdown widget"""
        return forms.Select(attrs={
            'class': "chosen-select",
            'data-placeholder': _('Group names'),
            'style': "width: 100%", 
        })


class MultiGroupSearchForm(SearchFieldForm):
    """Base class for searching by several groups"""
    multi_values = True

    def __init__(self, *args, **kwargs):
        super(MultiGroupSearchForm, self).__init__(*args, **kwargs)
        queryset = models.Group.objects.all()
        try:
            queryset = queryset.extra(select={'lower_name': 'lower(name)'}).order_by('lower_name')
        except Exception: #pylint: disable=broad-except
            queryset = queryset.order_by('name')
        kwargs = kwargs or {}
        kwargs.setdefault('required', True)
        widget = self._get_widget()
        if widget:
            kwargs['widget'] = widget
        field = forms.MultipleChoiceField(
            choices=[(x.id, '{0}'.format(x)) for x in queryset.all()], label=self.label, **kwargs)
        self._add_field(field)
        
    def get_values(self):
        """values"""
        if type(self.value) == list:
            values = self.value
        else:
            values = [self.value]
        return values
    
    def _get_widget(self):
        """customize widget"""
        return forms.SelectMultiple(attrs={
            'class': "chosen-select",
            'data-placeholder': _('Select Group names'),
            'style': "width: 100%", 
        })


class GroupsMemberOfAllSearchForm(MultiGroupSearchForm):
    """members of all groups"""
    name = 'all_groups'
    label = _('Members of all groups')
    
    def get_queryset(self, queryset):
        """queryset"""
        for group_id in self.get_values():
            group_queryset = ((Q(entity__isnull=False) & Q(entity__group__id=group_id)) | Q(group__id=group_id))
            queryset = queryset.filter(group_queryset)
        return queryset


class GroupsMemberOfAnySearchForm(MultiGroupSearchForm):
    """member of one of the groups"""
    name = 'any_groups'
    label = _('Members of at least one group')
    multi_values = True
    
    def get_queryset(self, queryset):
        """queryset"""
        group_queryset = Q(id=0)
        for group_id in self.get_values():
            current_queryset = (Q(entity__isnull=False) & Q(entity__group__id=group_id)) | Q(group__id=group_id)
            group_queryset = group_queryset | current_queryset
        queryset = queryset.filter(group_queryset)
        return queryset


class GroupsMemberOfNoneSearchForm(MultiGroupSearchForm):
    """not member of any of the groups"""
    name = 'none_groups'
    label = _('Member of none of these groups')
    multi_values = True
    
    def get_queryset(self, queryset):
        """queryset"""
        for group_id in self.get_values():
            group_queryset = ((Q(entity__isnull=False) & Q(entity__group__id=group_id)) | Q(group__id=group_id))
            queryset = queryset.exclude(group_queryset)
        return queryset


class NotInGroupSearchForm(SearchFieldForm):
    """not in group"""
    name = 'not_in_group'
    label = _('Not in group')
    
    def __init__(self, *args, **kwargs):
        super(NotInGroupSearchForm, self).__init__(*args, **kwargs)
        queryset = models.Group.objects.all()
        try:
            queryset = queryset.extra(select={'lower_name': 'lower(name)'}).order_by('lower_name')
        except Exception:  # pylint: disable=broad-except
            queryset = queryset.order_by('name')
        field = forms.ModelChoiceField(
            queryset,
            label=self.label,
            widget=GroupAutoComplete(attrs={'placeholder': _('Enter part of the group name'), 'size': '80'})
        )
        self._add_field(field)
    
    def get_lookup(self):
        """lookup"""
        return [~Q(entity__group__id=self.value), ~Q(group__id=self.value)]


class ContactAgeSearchForm(SearchFieldForm):
    """search by age"""
    name = 'contact_age'
    label = _('Contact age')
    
    def __init__(self, *args, **kwargs):
        super(ContactAgeSearchForm, self).__init__(*args, **kwargs)
        field = forms.CharField(label=self.label, initial='0 100')
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        ages = [int(x) for x in self.value.split()]
        dt_from = date.today() - timedelta(days=ages[1]*365.25)
        dt_to = date.today() - timedelta(days=ages[0]*365.25)
        return {'birth_date__gte': dt_from, 'birth_date__lte': dt_to}


class ContactAcceptSubscriptionSearchForm(SearchFieldForm):
    """by accept subscrition"""
    name = 'accept_subscription'
    label = _('Accept subscription to')
    
    def __init__(self, *args, **kwargs):
        super(ContactAcceptSubscriptionSearchForm, self).__init__(*args, **kwargs)
        queryset = models.SubscriptionType.objects.all()
        field = forms.ModelChoiceField(queryset, label=self.label)
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return {'subscription__subscription_type__id': self.value, 'subscription__accept_subscription': True}


class ContactRefuseSubscriptionSearchForm(ContactAcceptSubscriptionSearchForm):
    """by refuse subscription"""
    name = 'refuse_subscription'
    label = _('Refuse subscription to')
    
    def get_lookup(self):
        """lookup"""
        return None
        
    def get_exclude_lookup(self):
        """exclude lookup"""
        return super(ContactRefuseSubscriptionSearchForm, self).get_lookup()


class SecondarySearchForm(SearchFieldForm):
    """by secondary contact"""
    name = 'secondary_contact'
    label = _('Secondary contact')
    
    def __init__(self, *args, **kwargs):
        super(SecondarySearchForm, self).__init__(*args, **kwargs)
        choices = ((1, _('Include')), (0, _('Only')),)
        field = forms.ChoiceField(choices=choices, label=self.label)
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        value = int(self.value)
        if value == 1:
            #the lookup 'main_contact' will be removed by the search form
            return {}
        elif value == 0:
            return {'main_contact': False}


class ContactHasLeft(SearchFieldForm):
    """contact who left"""
    name = 'contact_has_left'
    label = _('Contact has left')
    
    def __init__(self, *args, **kwargs):
        super(ContactHasLeft, self).__init__(*args, **kwargs)
        choices = ((0, _('Only')), (1, _('Include')),)
        field = forms.ChoiceField(choices=choices, label=self.label)
        self._add_field(field)
            
    def get_lookup(self):
        """lookup"""
        value = int(self.value)
        if value == 1:
            #the lookup 'has_left' will be removed by the search form
            return {}
        elif value == 0:
            return {'has_left': True}


class ContactRoleSearchForm(SearchFieldForm):
    """by role"""
    name = 'contact_role'
    label = _('Contact role')
    
    def __init__(self, *args, **kwargs):
        super(ContactRoleSearchForm, self).__init__(*args, **kwargs)
        queryset = models.EntityRole.objects.all().order_by('name')
        field = forms.ModelChoiceField(queryset, label=self.label)
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return {'role': self.value}


class EmailSearchForm(SearchFieldForm):
    """by email"""
    name = 'contact_entity_email'
    label = _('Email')
    
    def __init__(self, *args, **kwargs):
        super(EmailSearchForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of the email of a contact or an entity')})
        )
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return Q(entity__email__icontains=self.value) | Q(email__icontains=self.value)


class ContactHasEmail(YesNoSearchFieldForm):
    """by has email"""
    name = 'contact_has_email'
    label = _('Contact has email')
            
    def get_lookup(self):
        """lookup"""
        has_no_email = (Q(email='') & Q(entity__email=''))
        if self.is_yes():
            return ~has_no_email
        else:
            return has_no_email


class ContactHasPersonalEmail(YesNoSearchFieldForm):
    """by has an email set on the contact (ignore if set on entity)"""
    name = 'contact_has_personal_email'
    label = _('Contact has pesonal email')
            
    def get_lookup(self):
        """queryset"""
        has_no_email = Q(email='')
        if self.is_yes():
            return ~has_no_email
        else:
            return has_no_email


class UnknownContact(YesNoSearchFieldForm):
    """no name"""
    name = 'unknown_contact'
    label = _('Unknown contacts')
        
    def get_lookup(self):
        """lookup"""
        unknown_contact = (Q(firstname='') & Q(lastname=''))
        if self.is_yes():
            return unknown_contact
        else:
            return ~unknown_contact


class ActionTypeSearchForm(SearchFieldForm):
    """by type of action"""
    name = 'action_type'
    label = _('Action type')
    is_action_form = True

    def __init__(self, *args, **kwargs):
        super(ActionTypeSearchForm, self).__init__(*args, **kwargs)
        queryset = models.ActionType.objects.all().order_by('name')
        field = forms.ModelChoiceField(queryset, label=self.label)
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return Q(type=self.value)


class ActionNameSearchForm(SearchFieldForm):
    """by subject of action"""
    name = 'action_name'
    label = _('Action subject')
    is_action_form = True

    def __init__(self, *args, **kwargs):
        super(ActionNameSearchForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(attrs={'placeholder': _('enter a part of the name of the searched action')})
        )
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return Q(subject__icontains=self.value)


class ExcludeActionNameSearchForm(ActionNameSearchForm):
    """by subject of action"""
    name = 'exclude_action_name'
    label = _(u'No Action with subject')
    is_action_form = True
    is_exclude_action_form = True


class RelationshipDateForm(TwoDatesForm):
    """by date of relationship"""
    name = 'relationship_date'
    label = _('Relationship date')
            
    def get_lookup(self):
        """lookup"""
        start_datetime, end_datetime = self._get_datetimes()
        return {'entity__relationship_date__gte': start_datetime, 'entity__relationship_date__lte': end_datetime}


class ContactNameSearchForm(SearchFieldForm):
    """by contact name"""
    name = 'contact_name'
    label = _('Contact name')
    
    def __init__(self, *args, **kwargs):
        super(ContactNameSearchForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of the name of the searched contact')})
        )
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return {'lastname__icontains': self.value}


class ContactLanguageSearchForm(SearchFieldForm):
    """by contact language"""
    name = 'contact_lang'
    label = _('Contact language')

    def __init__(self, *args, **kwargs):
        super(ContactLanguageSearchForm, self).__init__(*args, **kwargs)
        field = forms.ChoiceField(
            label=self.label,
            choices=get_language_choices()
        )
        self._add_field(field)
        field.required = False

    def get_lookup(self):
        """lookup"""
        return {'favorite_language': self.value}


class ContactFirstnameSearchForm(SearchFieldForm):
    """by firstname"""
    name = 'contact_firstname'
    label = _('Contact firstname')
    
    def __init__(self, *args, **kwargs):
        super(ContactFirstnameSearchForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of the firstname of the searched contact')})
        )
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return {'firstname__icontains': self.value}
    

class ContactNotesSearchForm(SearchFieldForm):
    """by notes"""
    name = 'contact_notes'
    label = _('Contact notes')
    
    def __init__(self, *args, **kwargs):
        super(ContactNotesSearchForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(attrs={'placeholder': _('Enter a part of a note of the searched contact')})
        )
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return {'notes__icontains': self.value}


class OpportunitySearchForm(SearchFieldForm):
    """by opportunity"""
    name = 'opportunity'
    label = _('Opportunity')
    is_action_form = True
    
    def __init__(self, *args, **kwargs):
        super(OpportunitySearchForm, self).__init__(*args, **kwargs)
        queryset = models.Opportunity.objects.all()
        field = forms.ModelChoiceField(queryset, label=self.label)
        self._add_field(field)
    
    def get_lookup(self):
        """lookup"""
        return Q(opportunity__id=self.value)
        

class OpportunityNameSearchForm(SearchFieldForm):
    """by opportunity name"""
    name = 'opportunity_name'
    label = _('Opportunity name')
    is_action_form = True
    
    def __init__(self, *args, **kwargs):
        super(OpportunityNameSearchForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label,
            widget=forms.TextInput(attrs={'placeholder': _('enter a part of the name of the searched opportunity')})
        )
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return Q(opportunity__name__icontains=self.value)


class NoSameAsForm(YesNoSearchFieldForm):
    """Allow same as contact in results"""
    name = 'no_same_as'
    label = _('Allow same-as')
    
    def get_lookup(self):
        """lookup"""
        pass
    
    def get_exclude_lookup(self):
        """exclude lookup"""
        pass
    
    def global_post_process(self, contacts):
        """this filters the full list of results"""
        if self.is_yes():
            return contacts
        else:
            same_as = {}
            filtered_contacts = []
            for contact in contacts:
                if contact.same_as:
                    if contact.same_as.id not in same_as:
                        same_as[contact.same_as.id] = contact
                        # filtered_contacts.append(contact.same_as.main_contact if contact.same_as else contact)
                        filtered_contacts.append(contact)
                    else:
                        # If the current contact has better priority (lower is better). Take this one
                        other_contact = same_as[contact.same_as.id]
                        if other_contact.same_as_priority > contact.same_as_priority:
                            contact_index = filtered_contacts.index(other_contact)
                            filtered_contacts[contact_index] = contact
                else:
                    filtered_contacts.append(contact)
            return filtered_contacts


class HasSameAsForm(SearchFieldForm):
    """all contacts with same as"""
    name = 'has_same_as'
    label = _('Has same-as')

    def __init__(self, *args, **kwargs):
        super(HasSameAsForm, self).__init__(*args, **kwargs)
        choices = ((0, _('No same as')), (1, _('Only top priority same-as')), (2, _('All priorities same-as')),)
        field = forms.ChoiceField(choices=choices, label=self.label)
        self._add_field(field)

    def get_lookup(self):
        """lookup"""
        value = int(self.value)

        if value == 0:
            return Q(same_as__isnull=True)
        elif value == 1:
            return Q(same_as__isnull=False, same_as_priority=1)
        elif value == 2:
            return Q(same_as__isnull=False)

        
class NoSameEmailForm(SearchFieldForm):
    """Allow same as contact in results"""
    name = 'no_same_email'
    label = _('Duplicate emails')

    def __init__(self, *args, **kwargs):
        super(NoSameEmailForm, self).__init__(*args, **kwargs)
        choices = ((0, _('Exclude')), (1, _('Only')),)
        field = forms.ChoiceField(choices=choices, label=self.label)
        self._add_field(field)
    
    def get_lookup(self):
        """lookup"""
        pass
    
    def get_exclude_lookup(self):
        """exclude lookup"""
        pass
    
    def global_post_process(self, contacts):
        """this filters the full list of results"""
        try:
            value = int(self.value)
        except ValueError:
            value = -1
        emails = {}
        filtered_contacts = []
        for contact in contacts:
            email = contact.get_email
            if email:
                if email not in emails:
                    emails[email] = contact
                    if value == 0:
                        filtered_contacts.append(contact)
                else:
                    if value == 1:
                        filtered_contacts.append(contact)
        return filtered_contacts


class DuplicatedContactsForm(SearchFieldForm):
    """Allow same as contact in results"""
    name = 'duplicated_contacts'
    label = _('Duplicated contacts')

    def __init__(self, *args, **kwargs):
        super(DuplicatedContactsForm, self).__init__(*args, **kwargs)
        choices = ((1, _('same firstname and lastname')), (2, _('same lastname')), )
        field = forms.ChoiceField(choices=choices, label=self.label)
        self._add_field(field)

    def get_lookup(self):
        """lookup"""
        pass

    def get_exclude_lookup(self):
        """exclude lookup"""
        pass

    def global_post_process(self, contacts):
        """this filters the full list of results"""
        keys = {}
        filtered_contacts = []
        try:
            value = int(self.value)
        except ValueError:
            value = 1
        for contact in contacts:
            if contact.lastname:  # ignore contacts without names
                key = contact.lastname if value == 2 else contact.fullname   # lastname only or fullname
                if key:
                    if key in keys:
                        filtered_contacts.append(contact)
                        filtered_contacts.append(keys[key])
                    else:
                        keys[key] = contact
        return filtered_contacts


class ContactsImportSearchForm(SearchFieldForm):
    """by import"""
    name = 'contact_import'
    label = _('Import')
    
    def __init__(self, *args, **kwargs):
        super(ContactsImportSearchForm, self).__init__(*args, **kwargs)
        queryset = models.ContactsImport.objects.order_by('name')
        field = forms.ModelChoiceField(queryset, label=self.label)
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        return {'imported_by': self.value}


class ByUserBaseFormSearchForm(SearchFieldForm):
    """by import"""

    def __init__(self, *args, **kwargs):
        super(ByUserBaseFormSearchForm, self).__init__(*args, **kwargs)
        queryset = User.objects.filter(is_staff=True).order_by('username')
        field = forms.ModelChoiceField(queryset, label=self.label)
        self._add_field(field)


class ContactsAndEntitiesModifiedBySearchForm(ByUserBaseFormSearchForm):
    """by import"""
    name = 'contacts_and_entities_modified_by'
    label = _('Contacts and entities modified by')

    def get_lookup(self):
        """lookup"""
        return Q(last_modified_by__id=self.value) | Q(entity__last_modified_by__id=self.value)


class ContactsModifiedBySearchForm(ByUserBaseFormSearchForm):
    """by import"""
    name = 'contacts_modified_by'
    label = _('Contacts and entities modified by')

    def get_lookup(self):
        """lookup"""
        return Q(last_modified_by__id=self.value)


class EntitiesModifiedBySearchForm(ByUserBaseFormSearchForm):
    """by import"""
    name = 'entities_modified_by'
    label = _('Contacts and entities modified by')

    def get_lookup(self):
        """lookup"""
        return Q(entity__last_modified_by__id=self.value)


class EntityByModifiedDate(TwoDatesForm):
    """every entity modified betwwen two dates"""
    name = 'entity_by_modified_date'
    label = _('Entity by modified date')

    def get_lookup(self):
        """lookup"""
        datetime1, datetime2 = self._get_datetimes()
        datetime2 += timedelta(1)
        return Q(entity__is_single_contact=False, entity__modified__gte=datetime1, entity__modified__lt=datetime2)


class ContactByModifiedDate(TwoDatesForm):
    """every contact modified between two dates"""
    name = 'contact_by_modified_date'
    label = _('Contact by modified date')

    def get_lookup(self):
        """lookup"""
        datetime1, datetime2 = self._get_datetimes()
        datetime2 += timedelta(1)
        return {'modified__gte': datetime1, 'modified__lt': datetime2}


class ContactsByCreationDate(TwoDatesForm):
    """by creation date"""
    name = 'contacts_by_creation_date'
    label = _('Contacts by creation date')
    
    def get_lookup(self):
        """lookup"""
        start_datetime, end_datetime = self._get_datetimes()
        return Q(created__gte=start_datetime, created__lte=end_datetime)


class EntitiesByCreationDate(TwoDatesForm):
    """by entity creation date"""
    name = 'entities_by_creation_date'
    label = _('Entities by creation date')
    
    def get_lookup(self):
        """lookup"""
        start_datetime, end_datetime = self._get_datetimes()
        return Q(
            entity__is_single_contact=False, entity__created__gte=start_datetime, entity__created__lte=end_datetime
        )
    

class ContactsAndEntitiesByChangeDate(TwoDatesForm):
    """by change date"""
    name = 'contacts_and_entities_by_change_date'
    label = _('Contacts and entities by change date')
    
    def get_lookup(self):
        """lookup"""
        start_datetime, end_datetime = self._get_datetimes()
        return Q(modified__gte=start_datetime, modified__lte=end_datetime) | \
            Q(created__gte=start_datetime, created__lte=end_datetime) | \
            Q(entity__modified__gte=start_datetime, entity__modified__lte=end_datetime) | \
            Q(entity__created__gte=start_datetime, entity__created__lte=end_datetime)
        

class ContactsRelationshipByType(SearchFieldForm):
    """by type of relationship"""
    name = 'contacts_by_relationship_type'
    label = _('Relationship type')
    
    def __init__(self, *args, **kwargs):
        super(ContactsRelationshipByType, self).__init__(*args, **kwargs)
        relationship_types = []
        for relationship_type in models.RelationshipType.objects.all():
            relationship_types.append((relationship_type.id, relationship_type.name))
            if relationship_type.reverse:
                relationship_types.append((-relationship_type.id, relationship_type.reverse))
        field = forms.CharField(label=self.label, widget=forms.Select(choices=relationship_types))
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        relationship_ids = []
        value, is_reverse = int(self.value), False
        if value < 0:
            value, is_reverse = -value, True
        relationship_type = models.RelationshipType.objects.get(id=value)
        for relationship in models.Relationship.objects.filter(relationship_type__id=relationship_type.id):
            if relationship_type.reverse:
                if is_reverse:
                    relationship_ids.append(relationship.contact2.id)
                else:
                    relationship_ids.append(relationship.contact1.id)
            else:
                relationship_ids += [relationship.contact1.id, relationship.contact2.id]
        relationship_ids = list(set(relationship_ids))  
        return Q(id__in=relationship_ids)
    

class ContactsRelationshipByDate(TwoDatesForm):
    """by relationship date"""
    name = 'contacts_by_relationship_dates'
    label = _('Relationship dates')
        
    def get_lookup(self):
        """lookup"""
        relationship_ids = []
        start_datetime, end_datetime = self._get_datetimes()
        end_datetime = end_datetime + timedelta(1)
        queryset = models.Relationship.objects.filter(created__gte=start_datetime, created__lt=end_datetime)
        for relationship_type in queryset:
            relationship_ids = [relationship_type.contact1.id, relationship_type.contact2.id]
        relationship_ids = list(set(relationship_ids))  
        return Q(id__in=relationship_ids)
    

class ContactWithCustomField(SearchFieldForm):
    """by contact with custom field"""
    name = 'contact_with_custom_field'
    label = _('Contacts with custom field')
    
    def __init__(self, *args, **kwargs):
        super(ContactWithCustomField, self).__init__(*args, **kwargs)
        custom_fields = []
        for custom_field in models.CustomField.objects.filter(model=models.CustomField.MODEL_CONTACT):
            custom_fields.append((custom_field.id, custom_field.label))
        field = forms.CharField(label=self.label, widget=forms.Select(choices=custom_fields))
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        value = int(self.value)
        custom_field = models.CustomField.objects.get(id=value, model=models.CustomField.MODEL_CONTACT)
        return Q(contactcustomfieldvalue__custom_field=custom_field)


class EntityWithCustomField(SearchFieldForm):
    """by entity custom field"""
    name = 'entity_with_custom_field'
    label = _('Entities with custom field')
    
    def __init__(self, *args, **kwargs):
        super(EntityWithCustomField, self).__init__(*args, **kwargs)
        custom_fields = []
        for custom_field in models.CustomField.objects.filter(model=models.CustomField.MODEL_ENTITY):
            custom_fields.append((custom_field.id, custom_field.label))
        field = forms.CharField(label=self.label, widget=forms.Select(choices=custom_fields))
        self._add_field(field)
        
    def get_lookup(self):
        """lookup"""
        value = int(self.value)
        custom_field = models.CustomField.objects.get(id=value, model=models.CustomField.MODEL_ENTITY)
        return Q(entity__entitycustomfieldvalue__custom_field=custom_field)
    

class CustomFieldBaseSearchForm(SearchFieldForm):
    """Base class for any Custom Field """
    custom_field_name = ''
    model = None
    widget = None

    def __init__(self, *args, **kwargs):
        super(CustomFieldBaseSearchForm, self).__init__(*args, **kwargs)

        self.custom_field = models.CustomField.objects.get_or_create(
            name=self.custom_field_name, model=self.model
        )[0]

        label = self.custom_field.label or self.custom_field.name

        field = forms.CharField(
            label=label,
            widget=self.widget
        )
        self._add_field(field)

    def get_lookup(self):
        """lookup"""
        value = self.value
        custom_field = self.custom_field

        if custom_field.model == models.CustomField.MODEL_ENTITY:
            queryset = models.EntityCustomFieldValue.objects.filter(custom_field=self.custom_field, value=value)
            entity_ids = [custom_field_value.entity.id for custom_field_value in queryset]
            return Q(entity__id__in=entity_ids)
        else:
            queryset = models.ContactCustomFieldValue.objects.filter(custom_field=self.custom_field, value=value)
            contact_ids = [custom_field_value.contact.id for custom_field_value in queryset]
            return Q(id__in=contact_ids)


class UnitTestEntityCustomFieldForm(CustomFieldBaseSearchForm):
    """Unit test for entity"""
    custom_field_name = 'ut_cf_entity'
    name = 'cf_ut_cf_entity'
    model = models.CustomField.MODEL_ENTITY
    label = "entity custom field"


class UnitTestContactCustomFieldForm(CustomFieldBaseSearchForm):
    """Unit test for contacts"""
    custom_field_name = 'ut_cf_contact'
    name = 'cf_ut_cf_contact'
    model = models.CustomField.MODEL_CONTACT
    label = "contact custom field"


class SortContacts(SearchFieldForm):
    """sort contacts"""
    name = 'sort'
    label = _('Sort contacts')
    contacts_display = True
    is_sort_form = True
    
    def __init__(self, *args, **kwargs):
        super(SortContacts, self).__init__(*args, **kwargs)
        choices = (
            ('entity', _('Entity name')),
            ('contact', _('Contact name')),
            ('name', _('Entity or Contact name')),
            ('zipcode', _('Zipcode')),
        ) 
        field = forms.CharField(
            label=self.label,
            widget=forms.Select(
                choices=choices,
                attrs={
                    'class': "chosen-select",
                    'style': "width: 100%",
                }
            )
        )
        self._add_field(field)
        self.default_country = get_default_country()

    def _sort_by_name(self, contact):
        return sort_by_name_callback(contact)

    def _sort_by_entity(self, contact):
        return sort_by_entity_callback(contact)

    def _sort_by_contact(self, contact):
        return sort_by_contact_callback(contact)

    def _sort_by_zipcode(self, contact):
        """sort by zipcode"""
        country = contact.get_country()
        value1 = contact.get_zip_code or '?'
        city = contact.get_city
        if not city:
            prefix1 = "C"
            prefix2 = value2 = "?"
        else:
            prefix1 = "A" if ((not country) or self.default_country.id == country.id) else "B"
            prefix2 = country.name if country else self.default_country.name
            value2 = city.name
        value3 = self._sort_by_name(contact)
        return prefix1, prefix2, value1, value2, value3
    
    def get_queryset(self, queryset):
        """queryset"""
        return queryset
    
    def global_post_process(self, contacts):
        """filter the final results"""
        callback = getattr(self, '_sort_by_{0}'.format(self.value), None)
        return sorted(contacts, key=callback)


class ContactWithEmailInGroupSearchForm(SearchFieldForm):
    """get contacts with someone with the same email in group"""
    name = 'email_in_group'
    label = _(u'Some with same email in group')

    def _get_widget(self):
        """customize widget: autocomplete"""
        return GroupAutoComplete(attrs={
            'placeholder': _(u'Enter part of the group name'), 'size': '80',
        })

    def __init__(self, *args, **kwargs):
        super(ContactWithEmailInGroupSearchForm, self).__init__(*args, **kwargs)
        queryset = models.Group.objects.all()
        try:
            queryset = queryset.extra(select={'lower_name': 'lower(name)'}).order_by('lower_name')
        except Exception:  # pylint: disable=broad-except
            queryset = queryset.order_by('name')
        kwargs = {}
        widget = self._get_widget()
        if widget:
            kwargs['widget'] = widget
        field = forms.ModelChoiceField(queryset, label=self.label, **kwargs)
        self._add_field(field)

    # def get_lookup(self):
    #     """lookup"""
    #     return Q(entity__group__id=self.value) | Q(group__id=self.value)

    def global_post_process(self, contacts):
        """filter the final results"""
        group_contacts = models.Contact.objects.filter(Q(entity__group__id=self.value) | Q(group__id=self.value))

        contacts_with_same_email_in_group = []
        for contact in contacts:
            email = contact.get_email
            if group_contacts.filter(Q(email=email) | Q(email='', entity__email=email)):
                contacts_with_same_email_in_group.append(contact)

        return contacts_with_same_email_in_group



# -*- coding: utf-8 -*-
"""unit testing"""

from __future__ import unicode_literals

from django.core.urlresolvers import reverse

from balafon.Crm.tests import BaseTestCase


class BoardPanelTest(BaseTestCase):
    """board"""
    def test_view_board_panel(self):
        """test view board"""

        response = self.client.get(reverse("crm_board_panel"))
        self.assertRedirects(response, reverse('users_favorites_list'))
